program make_gradient
use parameters
use variables
implicit none

	!======================================================================
	! local variables
	!======================================================================
	
	character(len=40) :: junk, fn, cit
	character(len=100) :: fn_grad, fn_preco_fw, fn_preco_rw, fn_output
	
	integer :: nx_min_loc, nx_max_loc
	integer :: ny_min_loc, ny_max_loc
	integer :: nz_min_loc, nz_max_loc
	integer :: ix_multi_loc, iy_multi_loc, iz_multi_loc
	integer :: rank, ind
	
	integer :: comp
	integer :: i, j, k, l, m, n, indx, indy, indz
	
	integer :: ix_min, ixn_min, ix_max, ixn_max, ixn_maxmax, ixn_minmin
	integer :: iy_min, iyn_min, iy_max, iyn_max, iyn_maxmax, iyn_minmin
	integer :: iz_min, izn_min, iz_max, izn_max, izn_maxmax, izn_minmin
	real :: res(1:2), dist, dist_h, dist_z, delta_h, delta_z
	real :: norm

	integer :: nbx, nby, nbz
	real, allocatable, dimension(:) :: bxco, byco, bzco
	real, allocatable, dimension(:) :: bxcom, bycom, bzcom
	real, allocatable, dimension(:,:,:) :: grad, preco_grad, mpreco_fw, mpreco_rw
	
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: u, preco_fw, preco_rw

	!======================================================================
	! initialisations
	!======================================================================
			
	if (lpd==2) then
		
		knots(0)=-1.0
		knots(1)=0.0
		knots(2)=1.0
		
		w(0)=1/3
		w(1)=4/3
		w(2)=1/3
		
	elseif (lpd==3) then
		
		knots(0)=-1.0
		knots(1)=-0.4472135954999579
		knots(2)=0.4472135954999579
		knots(3)=1.0
		
		w(0)=0.1666666666666
		w(1)=0.8333333333333
		w(2)=0.8333333333333
		w(3)=0.1666666666666
		
	elseif (lpd==4) then
		
		knots(0)=-1.0
		knots(1)=-0.6546536707079772
		knots(2)=0.0
		knots(3)=0.6546536707079772
		knots(4)=1.0
		
		w(0)=0.1000000000000
		w(1)=0.5444444444444
		w(2)=0.7111111111111
		w(3)=0.5444444444444
		w(4)=0.1000000000000
		
	elseif (lpd==5) then
		
		knots(0)=-1.0
		knots(1)=-0.7650553239294647
		knots(2)=-0.2852315164806451
		knots(3)=0.2852315164806451
		knots(4)=0.7650553239294647
		knots(5)=1.0
		
		w(0)=0.0666666666666667
		w(1)=0.3784749562978470
		w(2)=0.5548583770354862
		w(3)=0.5548583770354862
		w(4)=0.3784749562978470
		w(5)=0.0666666666666667
		
	elseif (lpd==6) then
		
		knots(0)=-1.0
		knots(1)=-0.8302238962785670
		knots(2)=-0.4688487934707142
		knots(3)=0.0
		knots(4)=0.4688487934707142
		knots(5)=0.8302238962785670
		knots(6)=1.0
		
		w(0)=0.0476190476190476
		w(1)=0.2768260473615659
		w(2)=0.4317453812098627
		w(3)=0.4876190476190476
		w(4)=0.4317453812098627
		w(5)=0.2768260473615659
		w(6)=0.0476190476190476
		
	elseif (lpd==7) then
		
		knots(0)=-1.0
		knots(1)=-0.8717401485096066
		knots(2)=-0.5917001814331423
		knots(3)=-0.2092992179024789
		knots(4)=0.2092992179024789
		knots(5)=0.5917001814331423
		knots(6)=0.8717401485096066
		knots(7)=1.0
		
		w(0)=0.0357142857142857
		w(1)=0.2107042271435061
		w(2)=0.3411226924835044
		w(3)=0.4124587946587038
		w(4)=0.4124587946587038
		w(5)=0.3411226924835044
		w(6)=0.2107042271435061
		w(7)=0.0357142857142857
		
	endif

	!======================================================================
	! read external input
	!======================================================================
	
	write(*,*) 'gradient (1, 2, 3): '
	read(*,*) comp
	write(*,*) 'iteration: '
	read(*,*) it
	write(*,*) 'horizontal correlation length (km): '
	read(*,*) delta_h
	delta_h=delta_h*1000
	write(*,*) 'vertical correlation length (km): '
	read(*,*) delta_z
	delta_z=delta_z*1000
	
	write(*,*) 'directory for sensitivity densities (e.g. "bla/bla/d/"): '
	read(*,*) fn_grad
	write(*,*) 'confirm directory: ', fn_grad
	
!	write(*,*) 'directory for forward pre-conditioner (e.g. "bla/bla/d/"): '
!	read(*,*) fn_preco_fw
!	write(*,*) 'confirm directory: ', fn_preco_fw
	
!	write(*,*) 'directory for backward pre-conditioner (e.g. "bla/bla/d/"): '
!	read(*,*) fn_preco_rw
!	write(*,*) 'confirm directory: ', fn_preco_rw
	
	write(*,*) 'directory for output (e.g. "bla/bla/d/"): '
	read(*,*) fn_output
	write(*,*) 'confirm directory: ', fn_output
	
	call int2str(it,cit)

	!======================================================================
	! read block coordinates
	!======================================================================
			
	open(unit=71,file='block_x',status='old',action='read')
	open(unit=72,file='block_y',status='old',action='read')
	open(unit=73,file='block_z',status='old',action='read')
	
		read(71,*) nbx
		read(72,*) nby
		read(73,*) nbz
		
		allocate(grad(1:nbx-1,1:nby-1,1:nbz-1))
		allocate(preco_grad(1:nbx-1,1:nby-1,1:nbz-1))
		allocate(mpreco_fw(1:nbx-1,1:nby-1,1:nbz-1))
		allocate(mpreco_rw(1:nbx-1,1:nby-1,1:nbz-1))
		allocate(bxco(1:nbx))
		allocate(byco(1:nby))
		allocate(bzco(1:nbz))
		allocate(bxcom(1:nbx-1))
		allocate(bycom(1:nby-1))
		allocate(bzcom(1:nbz-1))

	
		do i=1,nbx
			read(71,*) bxco(i)
			bxco(i)=bxco(i)*pi/180
		enddo
		do j=1,nby
			read(72,*) byco(j)
			byco(j)=byco(j)*pi/180
		enddo
		do k=1,nbz
			read(73,*) bzco(k)
			bzco(k)=bzco(k)*1000
		enddo
		
	close(unit=71)
	close(unit=72)
	close(unit=73)
				
	!======================================================================
	! read boxfile
	!======================================================================

	write(*,*) '----------------------------------'
	
	open(UNIT=15,FILE='../MODELS/boxfile',STATUS='OLD',ACTION='READ')
	
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	read(15,*) junk
	
	read(15,*) p
	read(15,*) px
	read(15,*) py
	read(15,*) pz
	
	!======================================================================
	! read individual box information
	!======================================================================
	
	read(15,*) junk	
	
	do ind=1,p
		
	   !==============================================================
	   ! read boxfile entries for one processor
	   !==============================================================
				
	    read(15,*) rank
	    read(15,*) ix_multi_loc, iy_multi_loc, iz_multi_loc
	    read(15,*) nx_min_loc, nx_max_loc
	    read(15,*) ny_min_loc, ny_max_loc
	    read(15,*) nz_min_loc, nz_max_loc
	    read(15,*) xmin, xmax
	    read(15,*) ymin, ymax
	    read(15,*) zmin, zmax
	    read(15,*) junk
		
	    nx=nx_max_loc-nx_min_loc
	    ny=ny_max_loc-ny_min_loc
	    nz=nz_max_loc-nz_min_loc
		
	    call int2str(rank-1,junk)
    
	    !===========================================================
	    ! make coordinates
	    !===========================================================
		
	    dx=(xmax-xmin)/(nx+1)		! width of one element in x direction
	    dy=(ymax-ymin)/(ny+1)		! width of one element in y direction
	    dz=(zmax-zmin)/(nz+1)		! width of one element in z direction
	
	    do i=0,nx
	    do n=0,lpd
	      x(i,n)=xmin+i*dx+0.5*(1+knots(n))*dx
	      sin_theta(i,:,:,n,:,:)=sin(x(i,n))
	    enddo
	    enddo

	    do j=0,ny
	    do n=0,lpd
	      y(j,n)=ymin+j*dy+0.5*(1+knots(n))*dy
	    enddo
	    enddo
	
	    do k=0,nz
	    do n=0,lpd
	      z(k,n)=zmax-k*dz-0.5*(1+knots(n))*dz
	    enddo
	    enddo
		
	    !===========================================================
	    ! load sensitivity kernel
	    !===========================================================
		
	    if (comp==1) then
		!fn='../DATA/OUTPUT/grad1_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
	    	fn=fn_grad(1:len_trim(fn_grad))//'grad1_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
	    elseif (comp==2) then
		!fn='../DATA/OUTPUT/grad2_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
	    	fn=fn_grad(1:len_trim(fn_grad))//'grad2_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
	    elseif (comp==3) then
		!fn='../DATA/OUTPUT/grad3_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
	    	fn=fn_grad(1:len_trim(fn_grad))//'grad3_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
	    endif
		
	    write(*,*) 'open file ', fn
	    open(unit=10,file=fn,action='read',form='unformatted')
	    read(10) u
	    close(unit=10)
		
	    !===========================================================
	    ! load forward preconditioner
	    !===========================================================
		
	    !fn='../DATA/OUTPUT/preco_fw_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
!	    fn=fn_preco_fw(1:len_trim(fn_preco_fw))//'preco_fw_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
	    
!	    write(*,*) 'open file ', fn
!	    open(unit=10,file=fn,action='read',form='unformatted')
!	    read(10) preco_fw
!	    close(unit=10)
	    
            !===========================================================
	    ! load backward preconditioner
	    !===========================================================
		
	    !fn='../DATA/OUTPUT/preco_rw_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
!	    fn=fn_preco_rw(1:len_trim(fn_preco_rw))//'preco_rw_'//junk(1:len_trim(junk))//'_'//cit(1:len_trim(cit))
	    
!	    write(*,*) 'open file ', fn
!	    open(unit=10,file=fn,action='read',form='unformatted')
!	    read(10) preco_rw
!	    close(unit=10)
				
	    !===============================================================
	    ! perform integration
	    !===============================================================
				
	    do indx=1,nbx-1
	    do indy=1,nby-1
	    do indz=1,nbz-1
		
	      !=======================================================
	      !- check if processor overlaps with this model block
	      !=======================================================
		
	      if ((bxco(indx+1)>=xmin) .and. (bxco(indx)<=xmax) .and. &
     	          (byco(indy+1)>=ymin) .and. (byco(indy)<=ymax) .and. &
		  (bzco(indz+1)>=zmin) .and. (bzco(indz)<=zmax)) then
				
		!===========================================================
		! find indices of integration limits
		!===========================================================
		
		!- x-direction
			
		if (bxco(indx)>=xmin) then
			res=minloc(abs(x-bxco(indx)))
			ix_min=res(1)-1
			ixn_min=res(2)-1
		else
			ix_min=0
			ixn_min=0
		endif
		
		if (bxco(indx+1)<=xmax) then
			res=minloc(abs(x-bxco(indx+1)))
			ix_max=res(1)-1
			ixn_max=res(2)-1
		else
			ix_max=nx
			ixn_max=lpd
		endif
		
		!- y-direction
			
		if (byco(indy)>=ymin) then
			res=minloc(abs(y-byco(indy)))
			iy_min=res(1)-1
			iyn_min=res(2)-1
		else
			iy_min=0
			iyn_min=0
		endif
		
		if (byco(indy+1)<=ymax) then
			res=minloc(abs(y-byco(indy+1)))
			iy_max=res(1)-1
			iyn_max=res(2)-1
		else
			iy_max=ny
			iyn_max=lpd
		endif
		
		!- z-direction
	
		if (bzco(indz)>=zmin) then
			res=minloc(abs(z-bzco(indz)))
			iz_max=res(1)-1
			izn_max=res(2)-1
		else
			iz_max=nz
			izn_max=lpd
		endif
		
		if (bzco(indz+1)<=zmax) then
			res=minloc(abs(z-bzco(indz+1)))
			iz_min=res(1)-1
			izn_min=res(2)-1
		else
			iz_min=0
			izn_min=0
		endif
		
		!===========================================================
		! perform integration for grad(indx,indy,indz) and precos
		!===========================================================
				
		do i=ix_min,ix_max
		do j=iy_min,iy_max
		do k=iz_min,iz_max
		
			if (i>ix_min) then 
				ixn_minmin=0
			else
				ixn_minmin=ixn_min
			endif
			
			if (j>iy_min) then 
				iyn_minmin=0
			else
				iyn_minmin=iyn_min
			endif
			
			if (k>iz_min) then 
				izn_minmin=0
			else
				izn_minmin=izn_min
			endif
			
				
			if (i<ix_max) then 
				ixn_maxmax=lpd
			else
				ixn_maxmax=ixn_max
			endif
			
			if (j<iy_max) then 
				iyn_maxmax=lpd
			else
				iyn_maxmax=iyn_max
			endif
			
			if (k<iz_max) then 
				izn_maxmax=lpd
			else
				izn_maxmax=izn_max
			endif
		
		do l=ixn_minmin,ixn_maxmax
	        do m=iyn_minmin,iyn_maxmax
		do n=izn_minmin,izn_maxmax
	          
			  grad(indx,indy,indz)=grad(indx,indy,indz)+ &
		  		w(l)*w(m)*w(n)*u(i,j,k,l,m,n)*sin_theta(i,j,k,l,m,n)*z(k,n)*z(k,n)*dx*dy*dz/8
				
!			  mpreco_fw(indx,indy,indz)=mpreco_fw(indx,indy,indz)+ &
!		  		w(l)*w(m)*w(n)*preco_fw(i,j,k,l,m,n)*sin_theta(i,j,k,l,m,n)*z(k,n)*z(k,n)*dx*dy*dz/8
				
!			  mpreco_rw(indx,indy,indz)=mpreco_rw(indx,indy,indz)+ &
!		  		w(l)*w(m)*w(n)*preco_rw(i,j,k,l,m,n)*sin_theta(i,j,k,l,m,n)*z(k,n)*z(k,n)*dx*dy*dz/8
				
		enddo
		enddo
		enddo
		enddo
		enddo
		enddo
				  	    
	      endif
		
	    enddo
	    enddo
	    enddo
         
	
	    
	enddo
		
	close(UNIT=15)

	!======================================================================
        ! Skalierung
        !======================================================================

	! Unter der Annahme, dass sich die Sensitivitätsdichten auf Kubikkilometer beziehen ...
		
	grad=grad*1e-9

	!======================================================================
        ! gradient preconditioning (multiplication with covariance matrix)
        !======================================================================
		
	!- mean box locations -------------------------------------------------

	do i=1,nbx-1
		bxcom(i)=(bxco(i)+bxco(i+1))/2
	enddo
	
	do j=1,nby-1
		bycom(j)=(byco(j)+byco(j+1))/2
	enddo
	
	do k=1,nbz-1
		bzcom(k)=(bzco(k)+bzco(k+1))/2
	enddo
	
	!- loop over all boxes ------------------------------------------------

	do i=1,nbx-1
	do j=1,nby-1
	do k=1,nbz-1
	
	  do l=(max(i-5,1)),(min(i+5,nbx-1))
	  do m=(max(j-5,1)),(min(j+5,nby-1))
	  do n=(max(k-5,1)),(min(k+5,nbz-1))
		
	    !- compute vertical and horizontal distances
	  
	    dist=(bzcom(k)*cos(bycom(j))*sin(bxcom(i))-bzcom(n)*cos(bycom(m))*sin(bxcom(l)))**2 &
    	        +(bzcom(k)*sin(bycom(j))*sin(bxcom(i))-bzcom(n)*sin(bycom(m))*sin(bxcom(l)))**2 &
	        +(bzcom(k)*cos(bxcom(i))-bzcom(n)*cos(bxcom(l)))**2
		
	    dist=sqrt(dist)
	    dist_z=abs(bzcom(k)-bzcom(k))
	    dist_h=sqrt(dist*dist-dist_z*dist_z)
				
	    !preco_grad(i,j,k)=preco_grad(i,j,k)+grad(l,m,n)*exp(-0.5*dist/(delta*delta))
	    
	    !- summation
	    
	    preco_grad(i,j,k)=preco_grad(i,j,k) &
	                     +grad(l,m,n)*exp(-0.5*(dist_h**2)/(delta_h**2))*exp(-0.5*(dist_z**2)/(delta_z**2))
			     
	  enddo
          enddo
	  enddo
	  
	  !- scale by number of elements that have been summed
	    
	    norm=real((min(i+5,nbx-1))-(max(i-5,1))) &
	        *real((min(j+5,nby-1))-(max(j-5,1))) &
		*real((min(k+5,nbz-1))-(max(k-5,1)))
		
	    preco_grad(i,j,k)=preco_grad(i,j,k)/norm
				
	enddo
	enddo
	enddo
	
	!======================================================================
        ! write output
        !======================================================================

	open(unit=12,file=fn_output(1:len_trim(fn_output))//'gradient',action='write')
	open(unit=13,file=fn_output(1:len_trim(fn_output))//'preco_gradient',action='write')
!	open(unit=14,file=fn_output(1:len_trim(fn_output))//'preco_fw',action='write')
!	open(unit=15,file=fn_output(1:len_trim(fn_output))//'preco_rw',action='write')

	write(12,*) nbx-1, nby-1, nbz-1
	write(13,*) nbx-1, nby-1, nbz-1
!	write(14,*) nbx-1, nby-1, nbz-1
!	write(15,*) nbx-1, nby-1, nbz-1
		
	do i=1,nbx-1
	do j=1,nby-1
	do k=nbz-1,1,-1
			
		write(12,*) grad(i,j,k)
		write(13,*) preco_grad(i,j,k)
!		write(14,*) mpreco_fw(i,j,k)
!		write(15,*) mpreco_rw(i,j,k)
	
	enddo
	enddo
	enddo
		
	close(unit=12)
	close(unit=13)	
!	close(unit=14)
!	close(unit=15)

		
        !======================================================================
        ! clean up
        !======================================================================
		
	deallocate(grad)
	deallocate(preco_grad)
	deallocate(mpreco_fw)
	deallocate(mpreco_rw)
	deallocate(bxco)
	deallocate(byco)
	deallocate(bzco)
	deallocate(bxcom)
	deallocate(bycom)
	deallocate(bzcom)
		

	
end program make_gradient


!==============================================================================
! subroutine for integer to string conversion
!==============================================================================

subroutine int2str(value, string)
implicit none

	integer, intent(in) :: value
	character(len=*), intent(inout) :: string
	
	character(len=10) :: c
	integer :: k, n, new_value, is
	real :: e
	
	e=1e9
	is=0
	
	if (value==0) then
		string(1:1)='0'
		string(2:10)=' '
	else
	
		new_value=value
	
		do k=1,10
			c(k:k)=char(floor(new_value/e)+48)
		
			if ((floor(new_value/e)==0) .and. (is==0)) then
				n=k
			else
				is=1
			endif
		
			new_value=new_value-e*floor(new_value/e)
			e=e/10
			string(k:k)=' '
		
		enddo
		
		string(1:10-n)=c(n+1:10)
		
	endif
	
	if (len(string)>10) then
		string(11:len(string))=' '
	endif

return
end subroutine int2str
