subroutine ses3d_evolution
use parameters
use variables
implicit none
include 'mpif.h'

	!======================================================================
	! local variables
	!======================================================================

	integer :: i,j,k,q, in, jn, kn
	real :: body_force, f
	real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd) :: L, M_dummy, Mdx, Mdy, Mdz

        real :: dummy_1, dummy_2, dummy_3, dummy
        real, dimension(0:nx_max,0:ny_max,0:nz_max) :: dummy_x, dummy_y, dummy_z
 
        !======================================================================
        ! computation of strong form strains
        !======================================================================

        exx=0.0                                          !- e_theta_theta
        eyy=0.0                                          !- e_phi_phi
        ezz=0.0                                          !- e_r_r
        
        exy=0.0                                          !- e_theta_phi
        exz=0.0                                          !- e_theta_r
        eyz=0.0                                          !- e_phi_r

        eyx=0.0                                          !- dummy
        ezx=0.0                                          !- dummy
        ezy=0.0                                          !- dummy

        dxux=0.0
        dyux=0.0
        dzux=0.0
        
        dxuy=0.0
        dyuy=0.0
        dzuy=0.0

        dxuz=0.0
        dyuz=0.0
        dzuz=0.0

        !----------------------------------------------------------------------
        !- derivatives of velocity field
        !----------------------------------------------------------------------

        do i=0,lpd
           do q=0,lpd

              dummy_3=-2*dl(q,i)/dz
              dummy_2=2*dl(q,i)/dy
              dummy_1=2*dl(q,i)/dx

              !----------------------------------------------------------------
              !- d(vz)/dz -----------------------------------------------------

              ezz(:,:,:,:,:,i)=ezz(:,:,:,:,:,i)+vz(:,:,:,:,:,q)*dummy_3

              !- d(vy)/dz -----------------------------------------------------

              ezy(:,:,:,:,:,i)=ezy(:,:,:,:,:,i)+vy(:,:,:,:,:,q)*dummy_3

              !- d(vx)/dz -----------------------------------------------------

              ezx(:,:,:,:,:,i)=ezx(:,:,:,:,:,i)+vx(:,:,:,:,:,q)*dummy_3

              !----------------------------------------------------------------
              !- d(vy)/dy -----------------------------------------------------

              eyy(:,:,:,:,i,:)=eyy(:,:,:,:,i,:)+vy(:,:,:,:,q,:)*dummy_2

              !- d(vx)/dy -----------------------------------------------------

              eyx(:,:,:,:,i,:)=eyx(:,:,:,:,i,:)+vx(:,:,:,:,q,:)*dummy_2

              !- d(vz)/dy -----------------------------------------------------

              eyz(:,:,:,:,i,:)=eyz(:,:,:,:,i,:)+vz(:,:,:,:,q,:)*dummy_2

              !----------------------------------------------------------------
              !- d(vx)/dx -----------------------------------------------------
              
              exx(:,:,:,i,:,:)=exx(:,:,:,i,:,:)+vx(:,:,:,q,:,:)*dummy_1

              !- d(vy)/dx -----------------------------------------------------

              exy(:,:,:,i,:,:)=exy(:,:,:,i,:,:)+vy(:,:,:,q,:,:)*dummy_1

              !- d(vz)/dx -----------------------------------------------------

              exz(:,:,:,i,:,:)=exz(:,:,:,i,:,:)+vz(:,:,:,q,:,:)*dummy_1

           enddo
        enddo

        !----------------------------------------------------------------------
        !- derivatives of displacement field
        !----------------------------------------------------------------------

        !- upper z-boundary ---------------------------------------------------

        if (iz_multi==pz) then

        do i=0,lpd
           do q=0,lpd

              dummy_3=-2*dl(q,i)/dz
              dummy_2=2*dl(q,i)/dy
              dummy_1=2*dl(q,i)/dx

              dzuz(:,:,0:pml-1,:,:,i)=dzuz(:,:,0:pml-1,:,:,i)+uz(:,:,0:pml-1,:,:,q)*dummy_3
              dzuy(:,:,0:pml-1,:,:,i)=dzuy(:,:,0:pml-1,:,:,i)+uy(:,:,0:pml-1,:,:,q)*dummy_3
              dzux(:,:,0:pml-1,:,:,i)=dzux(:,:,0:pml-1,:,:,i)+ux(:,:,0:pml-1,:,:,q)*dummy_3

              dyuy(:,:,0:pml-1,:,i,:)=dyuy(:,:,0:pml-1,:,i,:)+uy(:,:,0:pml-1,:,q,:)*dummy_2
              dyux(:,:,0:pml-1,:,i,:)=dyux(:,:,0:pml-1,:,i,:)+ux(:,:,0:pml-1,:,q,:)*dummy_2
              dyuz(:,:,0:pml-1,:,i,:)=dyuz(:,:,0:pml-1,:,i,:)+uz(:,:,0:pml-1,:,q,:)*dummy_2

              dxux(:,:,0:pml-1,i,:,:)=dxux(:,:,0:pml-1,i,:,:)+ux(:,:,0:pml-1,q,:,:)*dummy_1
              dxuy(:,:,0:pml-1,i,:,:)=dxuy(:,:,0:pml-1,i,:,:)+uy(:,:,0:pml-1,q,:,:)*dummy_1
              dxuz(:,:,0:pml-1,i,:,:)=dxuz(:,:,0:pml-1,i,:,:)+uz(:,:,0:pml-1,q,:,:)*dummy_1

           enddo
        enddo

        endif

        !- lower z-boundary ---------------------------------------------------

        if (iz_multi==1) then

        do i=0,lpd
           do q=0,lpd

              dummy_3=-2*dl(q,i)/dz
              dummy_2=2*dl(q,i)/dy
              dummy_1=2*dl(q,i)/dx

              dzuz(:,:,nz-pml+1:nz,:,:,i)=dzuz(:,:,nz-pml+1:nz,:,:,i)+uz(:,:,nz-pml+1:nz,:,:,q)*dummy_3
              dzuy(:,:,nz-pml+1:nz,:,:,i)=dzuy(:,:,nz-pml+1:nz,:,:,i)+uy(:,:,nz-pml+1:nz,:,:,q)*dummy_3
              dzux(:,:,nz-pml+1:nz,:,:,i)=dzux(:,:,nz-pml+1:nz,:,:,i)+ux(:,:,nz-pml+1:nz,:,:,q)*dummy_3

              dyuy(:,:,nz-pml+1:nz,:,i,:)=dyuy(:,:,nz-pml+1:nz,:,i,:)+uy(:,:,nz-pml+1:nz,:,q,:)*dummy_2
              dyux(:,:,nz-pml+1:nz,:,i,:)=dyux(:,:,nz-pml+1:nz,:,i,:)+ux(:,:,nz-pml+1:nz,:,q,:)*dummy_2
              dyuz(:,:,nz-pml+1:nz,:,i,:)=dyuz(:,:,nz-pml+1:nz,:,i,:)+uz(:,:,nz-pml+1:nz,:,q,:)*dummy_2

              dxux(:,:,nz-pml+1:nz,i,:,:)=dxux(:,:,nz-pml+1:nz,i,:,:)+ux(:,:,nz-pml+1:nz,q,:,:)*dummy_1
              dxuy(:,:,nz-pml+1:nz,i,:,:)=dxuy(:,:,nz-pml+1:nz,i,:,:)+uy(:,:,nz-pml+1:nz,q,:,:)*dummy_1
              dxuz(:,:,nz-pml+1:nz,i,:,:)=dxuz(:,:,nz-pml+1:nz,i,:,:)+uz(:,:,nz-pml+1:nz,q,:,:)*dummy_1

           enddo
        enddo

        endif

        !- left x-boundary ---------------------------------------------------

        if (ix_multi==1) then

        do i=0,lpd
           do q=0,lpd

              dummy_3=-2*dl(q,i)/dz
              dummy_2=2*dl(q,i)/dy
              dummy_1=2*dl(q,i)/dx

              dzuz(0:pml-1,:,:,:,:,i)=dzuz(0:pml-1,:,:,:,:,i)+uz(0:pml-1,:,:,:,:,q)*dummy_3
              dzuy(0:pml-1,:,:,:,:,i)=dzuy(0:pml-1,:,:,:,:,i)+uy(0:pml-1,:,:,:,:,q)*dummy_3
              dzux(0:pml-1,:,:,:,:,i)=dzux(0:pml-1,:,:,:,:,i)+ux(0:pml-1,:,:,:,:,q)*dummy_3

              dyuy(0:pml-1,:,:,:,i,:)=dyuy(0:pml-1,:,:,:,i,:)+uy(0:pml-1,:,:,:,q,:)*dummy_2
              dyux(0:pml-1,:,:,:,i,:)=dyux(0:pml-1,:,:,:,i,:)+ux(0:pml-1,:,:,:,q,:)*dummy_2
              dyuz(0:pml-1,:,:,:,i,:)=dyuz(0:pml-1,:,:,:,i,:)+uz(0:pml-1,:,:,:,q,:)*dummy_2

              dxux(0:pml-1,:,:,i,:,:)=dxux(0:pml-1,:,:,i,:,:)+ux(0:pml-1,:,:,q,:,:)*dummy_1
              dxuy(0:pml-1,:,:,i,:,:)=dxuy(0:pml-1,:,:,i,:,:)+uy(0:pml-1,:,:,q,:,:)*dummy_1
              dxuz(0:pml-1,:,:,i,:,:)=dxuz(0:pml-1,:,:,i,:,:)+uz(0:pml-1,:,:,q,:,:)*dummy_1

           enddo
        enddo

        endif


        !- right x-boundary ---------------------------------------------------

        if (ix_multi==px) then

        do i=0,lpd
           do q=0,lpd

              dummy_3=-2*dl(q,i)/dz
              dummy_2=2*dl(q,i)/dy
              dummy_1=2*dl(q,i)/dx

              dzuz(nx-pml+1:nx,:,:,:,:,i)=dzuz(nx-pml+1:nx,:,:,:,:,i)+uz(nx-pml+1:nx,:,:,:,:,q)*dummy_3
              dzuy(nx-pml+1:nx,:,:,:,:,i)=dzuy(nx-pml+1:nx,:,:,:,:,i)+uy(nx-pml+1:nx,:,:,:,:,q)*dummy_3
              dzux(nx-pml+1:nx,:,:,:,:,i)=dzux(nx-pml+1:nx,:,:,:,:,i)+ux(nx-pml+1:nx,:,:,:,:,q)*dummy_3

              dyuy(nx-pml+1:nx,:,:,:,i,:)=dyuy(nx-pml+1:nx,:,:,:,i,:)+uy(nx-pml+1:nx,:,:,:,q,:)*dummy_2
              dyux(nx-pml+1:nx,:,:,:,i,:)=dyux(nx-pml+1:nx,:,:,:,i,:)+ux(nx-pml+1:nx,:,:,:,q,:)*dummy_2
              dyuz(nx-pml+1:nx,:,:,:,i,:)=dyuz(nx-pml+1:nx,:,:,:,i,:)+uz(nx-pml+1:nx,:,:,:,q,:)*dummy_2

              dxux(nx-pml+1:nx,:,:,i,:,:)=dxux(nx-pml+1:nx,:,:,i,:,:)+ux(nx-pml+1:nx,:,:,q,:,:)*dummy_1
              dxuy(nx-pml+1:nx,:,:,i,:,:)=dxuy(nx-pml+1:nx,:,:,i,:,:)+uy(nx-pml+1:nx,:,:,q,:,:)*dummy_1
              dxuz(nx-pml+1:nx,:,:,i,:,:)=dxuz(nx-pml+1:nx,:,:,i,:,:)+uz(nx-pml+1:nx,:,:,q,:,:)*dummy_1

           enddo
        enddo

        endif

        !- left y-boundary ---------------------------------------------------

        if (iy_multi==1) then

        do i=0,lpd
           do q=0,lpd

              dummy_3=-2*dl(q,i)/dz
              dummy_2=2*dl(q,i)/dy
              dummy_1=2*dl(q,i)/dx

              dzuz(:,0:pml-1,:,:,:,i)=dzuz(:,0:pml-1,:,:,:,i)+uz(:,0:pml-1,:,:,:,q)*dummy_3
              dzuy(:,0:pml-1,:,:,:,i)=dzuy(:,0:pml-1,:,:,:,i)+uy(:,0:pml-1,:,:,:,q)*dummy_3
              dzux(:,0:pml-1,:,:,:,i)=dzux(:,0:pml-1,:,:,:,i)+ux(:,0:pml-1,:,:,:,q)*dummy_3

              dyuy(:,0:pml-1,:,:,i,:)=dyuy(:,0:pml-1,:,:,i,:)+uy(:,0:pml-1,:,:,q,:)*dummy_2
              dyux(:,0:pml-1,:,:,i,:)=dyux(:,0:pml-1,:,:,i,:)+ux(:,0:pml-1,:,:,q,:)*dummy_2
              dyuz(:,0:pml-1,:,:,i,:)=dyuz(:,0:pml-1,:,:,i,:)+uz(:,0:pml-1,:,:,q,:)*dummy_2

              dxux(:,0:pml-1,:,i,:,:)=dxux(:,0:pml-1,:,i,:,:)+ux(:,0:pml-1,:,q,:,:)*dummy_1
              dxuy(:,0:pml-1,:,i,:,:)=dxuy(:,0:pml-1,:,i,:,:)+uy(:,0:pml-1,:,q,:,:)*dummy_1
              dxuz(:,0:pml-1,:,i,:,:)=dxuz(:,0:pml-1,:,i,:,:)+uz(:,0:pml-1,:,q,:,:)*dummy_1

           enddo
        enddo

        endif

        !- right y-boundary ---------------------------------------------------

        if (iy_multi==py) then

        do i=0,lpd
           do q=0,lpd

              dummy_3=-2*dl(q,i)/dz
              dummy_2=2*dl(q,i)/dy
              dummy_1=2*dl(q,i)/dx

              dzuz(:,ny-pml+1:ny,:,:,:,i)=dzuz(:,ny-pml+1:ny,:,:,:,i)+uz(:,ny-pml+1:ny,:,:,:,q)*dummy_3
              dzuy(:,ny-pml+1:ny,:,:,:,i)=dzuy(:,ny-pml+1:ny,:,:,:,i)+uy(:,ny-pml+1:ny,:,:,:,q)*dummy_3
              dzux(:,ny-pml+1:ny,:,:,:,i)=dzux(:,ny-pml+1:ny,:,:,:,i)+ux(:,ny-pml+1:ny,:,:,:,q)*dummy_3

              dyuy(:,ny-pml+1:ny,:,:,i,:)=dyuy(:,ny-pml+1:ny,:,:,i,:)+uy(:,ny-pml+1:ny,:,:,q,:)*dummy_2
              dyux(:,ny-pml+1:ny,:,:,i,:)=dyux(:,ny-pml+1:ny,:,:,i,:)+ux(:,ny-pml+1:ny,:,:,q,:)*dummy_2
              dyuz(:,ny-pml+1:ny,:,:,i,:)=dyuz(:,ny-pml+1:ny,:,:,i,:)+uz(:,ny-pml+1:ny,:,:,q,:)*dummy_2

              dxux(:,ny-pml+1:ny,:,i,:,:)=dxux(:,ny-pml+1:ny,:,i,:,:)+ux(:,ny-pml+1:ny,:,q,:,:)*dummy_1
              dxuy(:,ny-pml+1:ny,:,i,:,:)=dxuy(:,ny-pml+1:ny,:,i,:,:)+uy(:,ny-pml+1:ny,:,q,:,:)*dummy_1
              dxuz(:,ny-pml+1:ny,:,i,:,:)=dxuz(:,ny-pml+1:ny,:,i,:,:)+uz(:,ny-pml+1:ny,:,q,:,:)*dummy_1

           enddo
        enddo

        endif

        exx=exx+(prof_z+prof_y)*dxux*ispml
        eyy=eyy+(prof_x+prof_z)*dyuy*ispml
        ezz=ezz+(prof_x+prof_y)*dzuz*ispml
        
        exy=exy+(prof_y+prof_z)*dxuy*ispml
        eyx=eyx+(prof_x+prof_z)*dyux*ispml
        
        exz=exz+(prof_y+prof_z)*dxuz*ispml
        ezx=ezx+(prof_x+prof_y)*dzux*ispml
        
        eyz=eyz+(prof_x+prof_z)*dyuz*ispml
        ezy=ezy+(prof_x+prof_y)*dzuy*ispml

        !- build strain rates --------------------------------------------------

        ! ezz=e_theta_theta is already done
    
        eyy(0:nx,0:ny,0:nz,:,:,:)=(eyy(0:nx,0:ny,0:nz,:,:,:)/sin_theta(0:nx,0:ny,0:nz,:,:,:) &
             +vz(0:nx,0:ny,0:nz,:,:,:)+vx(0:nx,0:ny,0:nz,:,:,:)*cot_theta(0:nx,0:ny,0:nz,:,:,:))/r(0:nx,0:ny,0:nz,:,:,:)

        exx(0:nx,0:ny,0:nz,:,:,:)=(exx(0:nx,0:ny,0:nz,:,:,:)+vz(0:nx,0:ny,0:nz,:,:,:))/r(0:nx,0:ny,0:nz,:,:,:)

        eyz(0:nx,0:ny,0:nz,:,:,:)=0.5*(ezy(0:nx,0:ny,0:nz,:,:,:)+ &
             (eyz(0:nx,0:ny,0:nz,:,:,:)/sin_theta(0:nx,0:ny,0:nz,:,:,:)-vy(0:nx,0:ny,0:nz,:,:,:))/r(0:nx,0:ny,0:nz,:,:,:))

        exz(0:nx,0:ny,0:nz,:,:,:)=0.5*(ezx(0:nx,0:ny,0:nz,:,:,:)+(exz(0:nx,0:ny,0:nz,:,:,:) &
             -vx(0:nx,0:ny,0:nz,:,:,:))/r(0:nx,0:ny,0:nz,:,:,:))

        exy(0:nx,0:ny,0:nz,:,:,:)=0.5*(eyx(0:nx,0:ny,0:nz,:,:,:)/sin_theta(0:nx,0:ny,0:nz,:,:,:) &
             -vy(0:nx,0:ny,0:nz,:,:,:)*cot_theta(0:nx,0:ny,0:nz,:,:,:)+exy(0:nx,0:ny,0:nz,:,:,:))/r(0:nx,0:ny,0:nz,:,:,:)
        
                                                    
	!======================================================================
	! make strong form stress rates
	!======================================================================
	
        ! - diagonal stresses -------------------------------------------------

        
        if (is_diss==1) then

           Mdx=0.0
           Mdy=0.0
           Mdz=0.0

           do k=1,nrdiss

              Mdx=Mdx+Mxx(k,:,:,:,:,:,:)
              Mdy=Mdy+Myy(k,:,:,:,:,:,:)
              Mdz=Mdz+Mzz(k,:,:,:,:,:,:)

           enddo

           M_dummy=Mdx+Mdy+Mdz

           L=(kappa-2*mu_tau/3)*(exx+eyy+ezz)-2*mu*M_dummy/3

           if (is_aniso==1) then

              sxx=L+2*mu_tau*exx+2*mu*Mdx+C*ezz+A*(eyy+exx)
              syy=L+2*mu_tau*eyy+2*mu*Mdy+C*ezz+A*(eyy+exx)
              szz=L+2*mu_tau*ezz+2*mu*Mdz+C*(eyy+exx)

           else

              sxx=L+2*mu_tau*exx+2*mu*Mdx
              syy=L+2*mu_tau*eyy+2*mu*Mdy
              szz=L+2*mu_tau*ezz+2*mu*Mdz

           endif
              
        else

           if (is_aniso==1) then
           
              L=lambda*(exx+eyy+ezz)
              sxx=L+2*mu*exx+C*ezz+A*(eyy+exx)
              syy=L+2*mu*eyy+C*ezz+A*(eyy+exx)
              szz=L+2*mu*ezz+C*(eyy+exx)

           else

              L=lambda*(exx+eyy+ezz)
              sxx=L+2*mu*exx
              syy=L+2*mu*eyy
              szz=L+2*mu*ezz

           endif

        endif

	!- off-diagonal stresses ----------------------------------------------
		
        if (is_diss==1) then

           Mdx=0.0
           Mdy=0.0
           Mdz=0.0
           
           do k=1,nrdiss

              Mdx=Mdx+Mxy(k,:,:,:,:,:,:)
              Mdy=Mdy+Mxz(k,:,:,:,:,:,:)
              Mdz=Mdz+Myz(k,:,:,:,:,:,:)

           enddo

           if (is_aniso==1) then

              sxy=2*mu_tau*exy+2*mu*Mdx
              sxz=2*mu_tau*exz+2*mu*Mdy+2*B*exz
              syz=2*mu_tau*eyz+2*mu*Mdz+2*B*eyz

           else

              sxy=2*mu_tau*exy+2*mu*Mdx
              sxz=2*mu_tau*exz+2*mu*Mdy
              syz=2*mu_tau*eyz+2*mu*Mdz

           endif

        else

           if (is_aniso==1) then

              sxy=2*mu*exy
              sxz=2*mu*exz+2*B*exz
              syz=2*mu*eyz+2*B*eyz

           else

              sxy=2*mu*exy
              sxz=2*mu*exz
              syz=2*mu*eyz

           endif

        endif
		
        !======================================================================
        ! march pml stresses
        !======================================================================

        sxx_pml=sxx_pml+dt*(sxx-ispml*prof_x*sxx_pml)
        syy_pml=syy_pml+dt*(syy-ispml*prof_y*syy_pml)
        szz_pml=szz_pml+dt*(szz-ispml*prof_z*szz_pml)

        sxy_pml=sxy_pml+dt*(sxy-ispml*prof_x*sxy_pml)
        syx_pml=syx_pml+dt*(sxy-ispml*prof_y*syx_pml)

        sxz_pml=sxz_pml+dt*(sxz-ispml*prof_x*sxz_pml)
        szx_pml=szx_pml+dt*(sxz-ispml*prof_z*szx_pml)

        syz_pml=syz_pml+dt*(syz-ispml*prof_y*syz_pml)
        szy_pml=szy_pml+dt*(syz-ispml*prof_z*szy_pml)

        !=====================================================================
        ! add moment tensor source
        !=====================================================================

!        if ((is_source>0) .and. (source_type==10)) then

!                sxx_pml(isx,isy,isz,isx_n,isy_n,isz_n)=sxx_pml(isx,isy,isz,isx_n,isy_n,isz_n)+MOM_xx*so(it)
!                syy_pml(isx,isy,isz,isx_n,isy_n,isz_n)=syy_pml(isx,isy,isz,isx_n,isy_n,isz_n)+MOM_yy*so(it)
!                szz_pml(isx,isy,isz,isx_n,isy_n,isz_n)=szz_pml(isx,isy,isz,isx_n,isy_n,isz_n)+MOM_zz*so(it)

!                sxy_pml(isx,isy,isz,isx_n,isy_n,isz_n)=sxy_pml(isx,isy,isz,isx_n,isy_n,isz_n)+MOM_xy*so(it)
!                syx_pml(isx,isy,isz,isx_n,isy_n,isz_n)=syx_pml(isx,isy,isz,isx_n,isy_n,isz_n)+MOM_xy*so(it)

!                sxz_pml(isx,isy,isz,isx_n,isy_n,isz_n)=sxz_pml(isx,isy,isz,isx_n,isy_n,isz_n)+MOM_xz*so(it)
!                szx_pml(isx,isy,isz,isx_n,isy_n,isz_n)=szx_pml(isx,isy,isz,isx_n,isy_n,isz_n)+MOM_xz*so(it)

!                syz_pml(isx,isy,isz,isx_n,isy_n,isz_n)=syz_pml(isx,isy,isz,isx_n,isy_n,isz_n)+MOM_yz*so(it)
!                szy_pml(isx,isy,isz,isx_n,isy_n,isz_n)=szy_pml(isx,isy,isz,isx_n,isy_n,isz_n)+MOM_yz*so(it)

!        endif
                
	!======================================================================

	sx=0.0
	sy=0.0
	sz=0.0
	
	!----------------------------------------------------------------------
	
	do i=0,lpd
	do j=0,lpd
	do k=0,lpd

        dummy_1=(2/dx)*Jac*w(j)*w(k)
        dummy_2=(2/dy)*Jac*w(i)*w(k)
        dummy_3=-(2/dz)*Jac*w(i)*w(j)

	do q=0,lpd

        dummy_x=w(q)*dl(i,q)*r(:,:,:,q,j,k)*sin_theta(:,:,:,q,j,k)*dummy_1
        dummy_y=w(q)*dl(j,q)*r(:,:,:,i,q,k)*dummy_2
        dummy_z=w(q)*dl(k,q)*r(:,:,:,i,j,q)*r(:,:,:,i,j,q)*sin_theta(:,:,:,i,j,q)*dummy_3

        sx(:,:,:,i,j,k)=sx(:,:,:,i,j,k) &
                       +szx_pml(:,:,:,i,j,q)*dummy_z &      
                       +syx_pml(:,:,:,i,q,k)*dummy_y &                                
                       +sxx_pml(:,:,:,q,j,k)*dummy_x                      
	
        sy(:,:,:,i,j,k)=sy(:,:,:,i,j,k) &
                       +szy_pml(:,:,:,i,j,q)*dummy_z &      
                       +syy_pml(:,:,:,i,q,k)*dummy_y &                                            
                       +sxy_pml(:,:,:,q,j,k)*dummy_x                       
	
        sz(:,:,:,i,j,k)=sz(:,:,:,i,j,k) &
                       +szz_pml(:,:,:,i,j,q)*dummy_z &      
                       +syz_pml(:,:,:,i,q,k)*dummy_y &                                            
                       +sxz_pml(:,:,:,q,j,k)*dummy_x                       
 			
	enddo

        sx(:,:,:,i,j,k)=sx(:,:,:,i,j,k)-w(i)*w(j)*w(k)*Jac*r(:,:,:,i,j,k)*sxz_pml(:,:,:,i,j,k)*sin_theta(:,:,:,i,j,k) &
                                       +w(i)*w(j)*w(k)*Jac*r(:,:,:,i,j,k)*syy_pml(:,:,:,i,j,k)*cos_theta(:,:,:,i,j,k)

!        sy(:,:,:,i,j,k)=sy(:,:,:,i,j,k)-w(i)*w(j)*w(k)*Jac*r(:,:,:,i,j,k)*syz_pml(:,:,:,i,j,k)*sin_theta(:,:,:,i,j,k) &
!                                       +w(i)*w(j)*w(k)*Jac*r(:,:,:,i,j,k)*syy_pml(:,:,:,i,j,k)*cos_theta(:,:,:,i,j,k)

! Hier hatten wir offenbar einen Fehler!

	sy(:,:,:,i,j,k)=sy(:,:,:,i,j,k)-w(i)*w(j)*w(k)*Jac*r(:,:,:,i,j,k)*syz_pml(:,:,:,i,j,k)*sin_theta(:,:,:,i,j,k) &
                                       -w(i)*w(j)*w(k)*Jac*r(:,:,:,i,j,k)*syx_pml(:,:,:,i,j,k)*cos_theta(:,:,:,i,j,k)

					       
        sz(:,:,:,i,j,k)=sz(:,:,:,i,j,k)+w(i)*w(j)*w(k)*Jac*r(:,:,:,i,j,k) &
             *(syy_pml(:,:,:,i,j,k)+sxx_pml(:,:,:,i,j,k))*sin_theta(:,:,:,i,j,k)

	enddo
	enddo
	enddo

	
	
        !======================================================================
        ! globalisation of local stresses
        !======================================================================

        do i=1,nx

           sx(i,:,:,0,:,:)=sx(i,:,:,0,:,:)+sx(i-1,:,:,lpd,:,:)
           sx(i-1,:,:,lpd,:,:)=sx(i,:,:,0,:,:);

           sy(i,:,:,0,:,:)=sy(i,:,:,0,:,:)+sy(i-1,:,:,lpd,:,:)
           sy(i-1,:,:,lpd,:,:)=sy(i,:,:,0,:,:);

           sz(i,:,:,0,:,:)=sz(i,:,:,0,:,:)+sz(i-1,:,:,lpd,:,:)
           sz(i-1,:,:,lpd,:,:)=sz(i,:,:,0,:,:);

        enddo

        do j=1,ny

           sx(:,j,:,:,0,:)=sx(:,j,:,:,0,:)+sx(:,j-1,:,:,lpd,:)
           sx(:,j-1,:,:,lpd,:)=sx(:,j,:,:,0,:)

           sy(:,j,:,:,0,:)=sy(:,j,:,:,0,:)+sy(:,j-1,:,:,lpd,:)
           sy(:,j-1,:,:,lpd,:)=sy(:,j,:,:,0,:)

           sz(:,j,:,:,0,:)=sz(:,j,:,:,0,:)+sz(:,j-1,:,:,lpd,:)
           sz(:,j-1,:,:,lpd,:)=sz(:,j,:,:,0,:)

        enddo

        do k=1,nz

           sx(:,:,k,:,:,0)=sx(:,:,k,:,:,0)+sx(:,:,k-1,:,:,lpd)
           sx(:,:,k-1,:,:,lpd)=sx(:,:,k,:,:,0)

           sy(:,:,k,:,:,0)=sy(:,:,k,:,:,0)+sy(:,:,k-1,:,:,lpd)
           sy(:,:,k-1,:,:,lpd)=sy(:,:,k,:,:,0)
           
           sz(:,:,k,:,:,0)=sz(:,:,k,:,:,0)+sz(:,:,k-1,:,:,lpd)
           sz(:,:,k-1,:,:,lpd)=sz(:,:,k,:,:,0)

        enddo

        call comm_S

	!======================================================================
	! add external body forces
	!======================================================================
	
	! Hier müsste man noch garantieren, dass die Kräfte auch an den doppelten Gitterpunkten ordentlich implementiert sind.

	if (is_source>0) then
	
                body_force=so(it)
        
		if (source_type==1) then
			sx(isx,isy,isz,isx_n,isy_n,isz_n)=sx(isx,isy,isz,isx_n,isy_n,isz_n)-body_force
		elseif (source_type==2) then
			sy(isx,isy,isz,isx_n,isy_n,isz_n)=sy(isx,isy,isz,isx_n,isy_n,isz_n)-body_force
		elseif (source_type==3) then
			sz(isx,isy,isz,isx_n,isy_n,isz_n)=sz(isx,isy,isz,isx_n,isy_n,isz_n)-body_force
		endif
		
	endif


        if ((is_source>0) .and. (source_type==10)) then

                sx=sx-(sf_dz*MOM_xz+sf_dx*MOM_xx/z(isz,isz_n) &
                      +sf_dy*MOM_xy/(z(isz,isz_n)*sin_theta(isx,isy,isz,isx_n,isy_n,isz_n)))*so(it)
                
                sy=sy-sf_dz*MOM_yz*so(it) &
                     -sf_dy*MOM_yy*so(it)/(z(isz,isz_n)*sin_theta(isx,isy,isz,isx_n,isy_n,isz_n)) &
                     -sf_dx*MOM_xy*so(it)/z(isz,isz_n)
                
                sz=sz-(sf_dz*MOM_zz+sf_dy*MOM_yz/(z(isz,isz_n)*sin_theta(isx,isy,isz,isx_n,isy_n,isz_n)) &
                                   +sf_dx*MOM_xz/z(isz,isz_n))*so(it) 
                
                
                sx(isx,isy,isz,isx_n,isy_n,isz_n)=sx(isx,isy,isz,isx_n,isy_n,isz_n)-3*MOM_xz*so(it)/z(isz,isz_n) &
                                                 -MOM_xx*so(it)*cot_theta(isx,isy,isz,isx_n,isy_n,isz_n)/z(isz,isz_n) &
                                                 +MOM_yy*so(it)*cot_theta(isx,isy,isz,isx_n,isy_n,isz_n)/z(isz,isz_n)
                
                sy(isx,isy,isz,isx_n,isy_n,isz_n)=sy(isx,isy,isz,isx_n,isy_n,isz_n)-3*MOM_yz*so(it)/z(isz,isz_n) &
                                                 -2*MOM_xy*so(it)*cot_theta(isx,isy,isz,isx_n,isy_n,isz_n)/z(isz,isz_n)
                
                sz(isx,isy,isz,isx_n,isy_n,isz_n)=sz(isx,isy,isz,isx_n,isy_n,isz_n)-2*MOM_zz*so(it)/z(isz,isz_n) &
                                                 -MOM_xz*so(it)*cot_theta(isx,isy,isz,isx_n,isy_n,isz_n)/z(isz,isz_n) &
                                                 +MOM_xx*so(it)/z(isz,isz_n) &
                                                 +MOM_yy*so(it)/z(isz,isz_n)
               

        endif

        !======================================================================
        ! make accelerations / divide weak stresses by mass matrix
        !======================================================================

        sx(0:nx,0:ny,0:nz,:,:,:)=-sx(0:nx,0:ny,0:nz,:,:,:)/MM(0:nx,0:ny,0:nz,:,:,:)
        sy(0:nx,0:ny,0:nz,:,:,:)=-sy(0:nx,0:ny,0:nz,:,:,:)/MM(0:nx,0:ny,0:nz,:,:,:)
        sz(0:nx,0:ny,0:nz,:,:,:)=-sz(0:nx,0:ny,0:nz,:,:,:)/MM(0:nx,0:ny,0:nz,:,:,:)

	!======================================================================
	! time extrapolation of the displacement field
	!======================================================================

        vx=vx+dt*(sx-ispml*prof*vx)
        vy=vy+dt*(sy-ispml*prof*vy)
        vz=vz+dt*(sz-ispml*prof*vz)

        ux=ux+dt*vx
        uy=uy+dt*vy
        uz=uz+dt*vz

        !======================================================================
        ! time extrapolation of memory variables
        !======================================================================

        if (is_diss==1) then

           do k=1,nrdiss

              L(0:nx,0:ny,0:nz,:,:,:)=dt*tau(0:nx,0:ny,0:nz,:,:,:)/(nrdiss*tau_sp(k,0:nx,0:ny,0:nz,:,:,:))

              Mxx(k,0:nx,0:ny,0:nz,:,:,:)=Mxx(k,0:nx,0:ny,0:nz,:,:,:)-dt*Mxx(k,0:nx,0:ny,0:nz,:,:,:)/tau_sp(k,0:nx,0:ny,0:nz,:,:,:) &
                                -L(0:nx,0:ny,0:nz,:,:,:)*exx(0:nx,0:ny,0:nz,:,:,:)

              Myy(k,0:nx,0:ny,0:nz,:,:,:)=Myy(k,0:nx,0:ny,0:nz,:,:,:)-dt*Myy(k,0:nx,0:ny,0:nz,:,:,:)/tau_sp(k,0:nx,0:ny,0:nz,:,:,:) &
                                -L(0:nx,0:ny,0:nz,:,:,:)*eyy(0:nx,0:ny,0:nz,:,:,:)

              Mzz(k,0:nx,0:ny,0:nz,:,:,:)=Mzz(k,0:nx,0:ny,0:nz,:,:,:)-dt*Mzz(k,0:nx,0:ny,0:nz,:,:,:)/tau_sp(k,0:nx,0:ny,0:nz,:,:,:) &
                                -L(0:nx,0:ny,0:nz,:,:,:)*ezz(0:nx,0:ny,0:nz,:,:,:)
              
              Mxy(k,0:nx,0:ny,0:nz,:,:,:)=Mxy(k,0:nx,0:ny,0:nz,:,:,:)-dt*Mxy(k,0:nx,0:ny,0:nz,:,:,:)/tau_sp(k,0:nx,0:ny,0:nz,:,:,:) &
                                -L(0:nx,0:ny,0:nz,:,:,:)*exy(0:nx,0:ny,0:nz,:,:,:)

              Mxz(k,0:nx,0:ny,0:nz,:,:,:)=Mxz(k,0:nx,0:ny,0:nz,:,:,:)-dt*Mxz(k,0:nx,0:ny,0:nz,:,:,:)/tau_sp(k,0:nx,0:ny,0:nz,:,:,:) &
                                -L(0:nx,0:ny,0:nz,:,:,:)*exz(0:nx,0:ny,0:nz,:,:,:)
              
              Myz(k,0:nx,0:ny,0:nz,:,:,:)=Myz(k,0:nx,0:ny,0:nz,:,:,:)-dt*Myz(k,0:nx,0:ny,0:nz,:,:,:)/tau_sp(k,0:nx,0:ny,0:nz,:,:,:) &
                                -L(0:nx,0:ny,0:nz,:,:,:)*eyz(0:nx,0:ny,0:nz,:,:,:)

           enddo

        endif

        !=======================================================================
        ! compute kinetic energy
        !=======================================================================

        E_kin=0.0
        L=(vx*vx+vy*vy+vz*vz)
        L(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)=L(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)/rhoinv(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)
        L(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)=L(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)*r(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd) &
             *r(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)*sin_theta(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)
        

        do i=0,nx
           do j=0,ny
              do k=0,nz

                 do in=0,lpd
                    do jn=0,lpd
                       do kn=0,lpd

                          E_kin=E_kin+L(i,j,k,in,jn,kn)*w(in)*w(jn)*w(kn)

                       enddo
                    enddo
                 enddo

              enddo
           enddo
        enddo

        E_kin=-0.5*E_kin*Jac

        call mpi_reduce(E_kin,E_kin_total,1,mpi_real,mpi_sum,0,mpi_comm_world,ierr)
        call mpi_bcast(E_kin_total,1,mpi_real,0,mpi_comm_world,ierr)

        ux=ux*taper
        uy=uy*taper
        uz=uz*taper

        vx=vx*taper
        vy=vy*taper
        vz=vz*taper

        sxx_pml=sxx_pml*taper
        syy_pml=syy_pml*taper
        szz_pml=szz_pml*taper
           
        sxy_pml=sxy_pml*taper
        syx_pml=syx_pml*taper

        szx_pml=szx_pml*taper
        sxz_pml=sxz_pml*taper

        syz_pml=syz_pml*taper
        szy_pml=szy_pml*taper
       
end subroutine ses3d_evolution
