!==============================================================================
! subroutine for the initialisation of space matrices, mass matrix, ...
!==============================================================================
! last modified: 17 April 2007, Andreas Fichtner
!==============================================================================

subroutine ses3d_init_grad
use parameters
use variables
implicit none
include 'mpif.h'

	!======================================================================
	! local variables
	!======================================================================

	character(len=5) :: s_idx
	character(len=20) :: dummy
	integer :: status(MPI_STATUS_SIZE)
	integer :: i, j, k, l, m, n, idx
        real :: delta_x, delta_y, delta_z, alpha
        real :: tol

        !=====================================================================

        itmem=0
        ispml=1.0
        tol=pi/(180*100)                !- receiver location tolerance
    
	!======================================================================
	
	call int2str(my_rank, dummy)
	
    	if (my_rank==0) then
    
	    write(*,*)'begin init '
    	    
	endif
	
	write(99,*)'begin init '
	write(99,*) '----------------------------------'

        !======================================================================
        ! elastic parameters and memory variables and gradients
        !======================================================================

        E_kin_total=0.0
        E_kin_total_old=0.0

        kappa=lambda+2*mu/3
	
	grad1=0.0
	grad2=0.0
	grad3=0.0

        if (is_diss==1) then

           mu_tau=(1+tau)*mu

           Mxx(:,:,:,:,:,:,:)=0.0
           Myy(:,:,:,:,:,:,:)=0.0
           Mzz(:,:,:,:,:,:,:)=0.0

           Mxy(:,:,:,:,:,:,:)=0.0
           Mxz(:,:,:,:,:,:,:)=0.0
           Myz(:,:,:,:,:,:,:)=0.0

        endif
        
	!======================================================================
	! determine collocation points (knots) and integration weights
	!======================================================================
	
	if (lpd==2) then
		
		knots(0)=-1.0
		knots(1)=0.0
		knots(2)=1.0
		
		w(0)=1/3
		w(1)=4/3
		w(2)=1/3
		
	elseif (lpd==3) then
		
		knots(0)=-1.0
		knots(1)=-0.4472135954999579
		knots(2)=0.4472135954999579
		knots(3)=1.0
		
		w(0)=0.1666666666666
		w(1)=0.8333333333333
		w(2)=0.8333333333333
		w(3)=0.1666666666666
		
	elseif (lpd==4) then
		
		knots(0)=-1.0
		knots(1)=-0.6546536707079772
		knots(2)=0.0
		knots(3)=0.6546536707079772
		knots(4)=1.0
		
		w(0)=0.1000000000000
		w(1)=0.5444444444444
		w(2)=0.7111111111111
		w(3)=0.5444444444444
		w(4)=0.1000000000000
		
	elseif (lpd==5) then
		
		knots(0)=-1.0
		knots(1)=-0.7650553239294647
		knots(2)=-0.2852315164806451
		knots(3)=0.2852315164806451
		knots(4)=0.7650553239294647
		knots(5)=1.0
		
		w(0)=0.0666666666666667
		w(1)=0.3784749562978470
		w(2)=0.5548583770354862
		w(3)=0.5548583770354862
		w(4)=0.3784749562978470
		w(5)=0.0666666666666667
		
	elseif (lpd==6) then
		
		knots(0)=-1.0
		knots(1)=-0.8302238962785670
		knots(2)=-0.4688487934707142
		knots(3)=0.0
		knots(4)=0.4688487934707142
		knots(5)=0.8302238962785670
		knots(6)=1.0
		
		w(0)=0.0476190476190476
		w(1)=0.2768260473615659
		w(2)=0.4317453812098627
		w(3)=0.4876190476190476
		w(4)=0.4317453812098627
		w(5)=0.2768260473615659
		w(6)=0.0476190476190476
		
	elseif (lpd==7) then
		
		knots(0)=-1.0
		knots(1)=-0.8717401485096066
		knots(2)=-0.5917001814331423
		knots(3)=-0.2092992179024789
		knots(4)=0.2092992179024789
		knots(5)=0.5917001814331423
		knots(6)=0.8717401485096066
		knots(7)=1.0
		
		w(0)=0.0357142857142857
		w(1)=0.2107042271435061
		w(2)=0.3411226924835044
		w(3)=0.4124587946587038
		w(4)=0.4124587946587038
		w(5)=0.3411226924835044
		w(6)=0.2107042271435061
		w(7)=0.0357142857142857
		
	endif

       
	!======================================================================
	! initialisation of LAGRANGE polynomial derivatives
	!======================================================================
	
	do i=0,lpd
		do j=0,lpd

			call dlgll(lpd,i,j,dl(i,j))

		enddo
	enddo
	
	!======================================================================
	! initialisation of the space matrices
	!======================================================================
	
	dx=(xmax-xmin)/(nx+1)		! width of one element in x direction
	dy=(ymax-ymin)/(ny+1)		! width of one element in y direction
	dz=(zmax-zmin)/(nz+1)		! width of one element in z direction
	
	write(99,*) 'width of one element in theta direction in deg: ', dx*180/pi
	write(99,*) 'width of one element in phi direction in deg: ', dy*180/pi
	write(99,*) 'width of one element in z direction in m: ', dz
		
	open(unit=101,file=ofd(1:len_trim(ofd))//'COORDINATES/xco_'//dummy(1:len_trim(dummy)),status='unknown')
	open(unit=102,file=ofd(1:len_trim(ofd))//'COORDINATES/yco_'//dummy(1:len_trim(dummy)),status='unknown')
	open(unit=103,file=ofd(1:len_trim(ofd))//'COORDINATES/zco_'//dummy(1:len_trim(dummy)),status='unknown')
	
	do i=0,nx
		do n=0,lpd
			x(i,n)=xmin+i*dx+0.5*(1+knots(n))*dx
                        sin_theta(i,:,:,n,:,:)=sin(x(i,n))
                        cot_theta(i,:,:,n,:,:)=cos(x(i,n))/sin(x(i,n))
                        cos_theta(i,:,:,n,:,:)=cos(x(i,n))
			write(101,*) x(i,n)
		enddo
	enddo

	do j=0,ny
		do n=0,lpd
			y(j,n)=ymin+j*dy+0.5*(1+knots(n))*dy
			write(102,*) y(j,n)
		enddo
	enddo
	
	do k=0,nz
		do n=0,lpd
			z(k,n)=zmax-k*dz-0.5*(1+knots(n))*dz
                        r(:,:,k,:,:,n)=z(k,n)
			write(103,*) z(k,n)
		enddo
	enddo
	
	close(unit=101)
	close(unit=102)
	close(unit=103)
	
	!======================================================================
	! initialisation of the JACOBIAN
	!======================================================================
	
	Jac=-dx*dy*dz/8

        !======================================================================
        ! initialisation of the mass matrix
        !======================================================================

        ! local mass matrix

        do i=0,nx
           do j=0,ny
              do k=0,nz
                 do l=0,lpd
                    do m=0,lpd
                       do n=0,lpd

                          MM(i,j,k,l,m,n)=w(l)*w(m)*w(n)*Jac*z(k,n)*z(k,n)*sin_theta(i,j,k,l,m,n)/rhoinv(i,j,k,l,m,n)

                       enddo
                    enddo
                 enddo
              enddo
           enddo
        enddo

       
        ! interchange of elements -> global mass matrix

        do i=1,nx
           MM(i,:,:,0,:,:)=MM(i,:,:,0,:,:)+MM(i-1,:,:,lpd,:,:)
           MM(i-1,:,:,lpd,:,:)=MM(i,:,:,0,:,:)
        enddo

        do j=1,ny
           MM(:,j,:,:,0,:)=MM(:,j,:,:,0,:)+MM(:,j-1,:,:,lpd,:)
           MM(:,j-1,:,:,lpd,:)=MM(:,j,:,:,0,:)
        enddo

        do k=1,nz
           MM(:,:,k,:,:,0)=MM(:,:,k,:,:,0)+MM(:,:,k-1,:,:,lpd)
           MM(:,:,k-1,:,:,lpd)=MM(:,:,k,:,:,0)
        enddo

        ! communication (globalisation)

        call comm_M
	
	!=============================================================================
        ! make local adjoint source locations and read adjoint source time functions
        !=============================================================================

        nr_adsrc=0

        write(99,*) '--------------------------------------'
        write(99,*) '- adjoint sources --------------------'
        write(99,*) '--------------------------------------'

        do k=1,nr_adsrc_global

                if (my_rank==0) then

                        write(*,*) ad_srcloc_global(:,k)

                endif

                if (my_rank==122) then

                        write(*,*) xmin, xmax, ymin, ymax, zmin, zmax

                endif

           if ((ad_srcloc_global(1,k)<=xmax) .and. (ad_srcloc_global(1,k)>=xmin) .and. &
               (ad_srcloc_global(2,k)<=ymax) .and. (ad_srcloc_global(2,k)>=ymin) .and. &
               (zmax_global-ad_srcloc_global(3,k)<=zmax) .and. (zmax_global-ad_srcloc_global(3,k)>=zmin)) then
              
              nr_adsrc=nr_adsrc+1

              ad_srcloc(1,nr_adsrc)=ad_srcloc_global(1,k)
              ad_srcloc(2,nr_adsrc)=ad_srcloc_global(2,k)
              ad_srcloc(3,nr_adsrc)=ad_srcloc_global(3,k)

              write(99,*) ad_srcloc(1,nr_adsrc), ad_srcloc(2,nr_adsrc), ad_srcloc(3,nr_adsrc)
              write(*,*) 'adjoint source in processor', my_rank

              call int2str(k,s_idx)

              open(unit=10,file='ad_src_x_'//s_idx(1:len_trim(s_idx)),action='read')
              open(unit=11,file='ad_src_y_'//s_idx(1:len_trim(s_idx)),action='read')
              open(unit=12,file='ad_src_z_'//s_idx(1:len_trim(s_idx)),action='read')
              
              do i=1,nt
                 
                 read(10,*) ad_src_x(nr_adsrc,i)
                 read(11,*) ad_src_y(nr_adsrc,i)
                 read(12,*) ad_src_z(nr_adsrc,i)

              enddo

           endif

        enddo


	!======================================================================
	! indices of adjoint source locations
	!======================================================================

        do idx=1,nr_adsrc

           !- search in x-direction

           delta_x=xmax-xmin

           do i=0,nx
              do k=0,lpd

                 if (abs(ad_srcloc(1,idx)-x(i,k))<delta_x) then

                    is(1,idx)=i
                    isn(1,idx)=k

                    delta_x=abs(ad_srcloc(1,idx)-x(i,k))

                 endif

              enddo
           enddo

           !- search in y-direction

           delta_y=ymax-ymin

           do i=0,ny
              do k=0,lpd

                 if (abs(ad_srcloc(2,idx)-y(i,k))<delta_y) then

                    is(2,idx)=i
                    isn(2,idx)=k

                    delta_y=abs(ad_srcloc(2,idx)-y(i,k))

                 endif

              enddo
           enddo

           !- search in z-direction

           delta_z=zmax-zmin

           do i=0,nz
              do k=0,lpd

                 if (abs(zmax_global-ad_srcloc(3,idx)-z(i,k))<delta_z) then

                    is(3,idx)=i
                    isn(3,idx)=k

                    delta_z=abs(zmax_global-ad_srcloc(3,idx)-z(i,k))

                 endif

              enddo
           enddo

           !- write info to logfile

	   write(99,*) ' Reverse Source location '
           write(99,*) is(1,idx), is(2,idx), is(3,idx)
           write(99,*) x(is(1,idx),isn(1,idx)), y(is(2,idx),isn(2,idx)), z(is(3,idx),isn(3,idx))

        enddo
	
     	
        !======================================================================
        ! initialise source field
        !======================================================================

        sf_dx=0.0
        sf_dy=0.0
        sf_dz=0.0


        !======================================================================
        ! PML damping profiles
        !======================================================================


        alpha=1.2

        prof_x=0.0
        prof_y=0.0
        prof_z=0.0
        
        !- upper z-boundary ---------------------------------------------------

!        if (iz_multi==pz) then

!           do k=0,pml-1
!              do n=0,lpd

!                 delta_z=(zmax-pml*dz-z(k,n))/(pml*dz)
!                 prof_z(0:nx,0:ny,k,0:lpd,0:lpd,n)=alpha*delta_z*delta_z

!              enddo
!           enddo

!        endif

        !- lower z-boundary

        if (iz_multi==1) then

           do k=(nz-pml+1),nz
              do n=0,lpd

                 delta_z=(zmin+pml*dz-z(k,n))/(pml*dz)
                 prof_z(0:nx,0:ny,k,0:lpd,0:lpd,n)=alpha*delta_z*delta_z

              enddo
           enddo

        endif

        !- left x-boundary
                 
        if (ix_multi==1) then

           do i=0,pml-1
              do n=0,lpd

                 delta_x=(xmin+pml*dx-x(i,n))/(pml*dx)
                 prof_x(i,0:ny,0:nz,n,0:lpd,0:lpd)=alpha*delta_x*delta_x

              enddo
           enddo

        endif

        !- right x-boundary

        if (ix_multi==px) then

           do i=(nx-pml+1),nx
              do n=0,lpd

                 delta_x=(xmax-dx*pml-x(i,n))/(pml*dx)
                 prof_x(i,0:ny,0:nz,n,0:lpd,0:lpd)=alpha*delta_x*delta_x

              enddo
           enddo

        endif

        !- left y-boundary
                 
        if (iy_multi==1) then

           do i=0,pml-1
              do n=0,lpd

                 delta_y=(ymin+pml*dy-y(i,n))/(pml*dy)
                 prof_y(0:nx,i,0:nz,0:lpd,n,0:lpd)=alpha*delta_y*delta_y

              enddo
           enddo

        endif

        !- right y-boundary

        if (iy_multi==py) then

           do i=(ny-pml+1),ny
              do n=0,lpd

                 delta_y=(ymax-dy*pml-y(i,n))/(pml*dy)
                 prof_y(0:nx,i,0:nz,0:lpd,n,0:lpd)=alpha*delta_y*delta_y

              enddo
           enddo

        endif

        !- normalize the corners

        prof=prof_x+prof_y+prof_z

        prof_z=2*prof_z/(1+prof/alpha)
        prof_y=2*prof_y/(1+prof/alpha)
        prof_x=2*prof_x/(1+prof/alpha)

        prof=prof_x+prof_y+prof_z
       
        taper=exp(-0.1*prof*prof)

        !===================================
        ! make boundary Q
        !===================================

        tau=(1-taper)*tau

        !- indices

        do i=1,pml
           pmlidx(i)=i-1
           pmlidy(i)=i-1
           pmlidz(i)=i-1
        enddo

        do i=(pml+1),(2*pml)
           pmlidx(i)=nx-2*pml+i
           pmlidy(i)=ny-2*pml+i
           pmlidz(i)=nz-2*pml+i
        enddo

        write(99,*) 'pml indices:'
        write(99,*) pmlidx
        write(99,*) pmlidy
        write(99,*) pmlidz

	!======================================================================
	! clean up
	!======================================================================

	if (my_rank==0) then

		write(*,*) 'end init'
	
	endif
	
	write(99,*) 'end init'
	write(99,*) '----------------------------------------'

	
end subroutine ses3d_init_grad
