!*****************************************************************************
!***************** Subroutine to load parameters *****************************
!*****************************************************************************

subroutine ses3d_input
use parameters
use variables
implicit none
include 'mpif.h'


    character(len=15) :: junk    ! dummy variable 
    integer :: pp, dummy
    integer :: status(MPI_STATUS_SIZE)

    character(len=4) :: test_name
    
    integer :: nx_min_loc, nx_max_loc
    integer :: ny_min_loc, ny_max_loc
    integer :: nz_min_loc, nz_max_loc
    
    integer :: ix_multi_loc, iy_multi_loc, iz_multi_loc
    
    real :: xmin_loc, xmax_loc
    real :: ymin_loc, ymax_loc
    real :: zmin_loc, zmax_loc
    
    integer :: i, j, k, l, m, n


    	write(99,*) 'begin input'
    
	!=====================================================================
	! read box file, only process 0, and send to other processes
	!=====================================================================
	
	if (my_rank==0) then
		
		write(*,*) '----------------------------------------'
		write(*,*) 'begin input'
		write(*,*) 'process 0 reading boxfile' 
		open(UNIT=15,FILE='../MODELS/boxfile',STATUS='OLD',ACTION='READ')
	
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		read(15,*) junk
		
		write(99,*) '----------------------------------'
		write(99,*) '- parallelisation parameters -----'
		write(99,*) '----------------------------------'
	
		read(15,*) pp
		write(99,*) 'total number of processors:', pp
		read(15,*) px
		write(99,*) 'number of processors in theta direction:', px
		read(15,*) py
		write(99,*) 'number of processors in phi direction:', py
		read(15,*) pz
		write(99,*) 'number of processors in z direction:', pz
		
		write(99,*) '----------------------------------'
		
		if (pp/=p) then
			
			write(*,*) 'incorrect number of processes'
			write(*,*) 'inconsistency in boxfile and s_mpirun'
			write(99,*) 'ERROR: incorrect number of processes'
			write(99,*) 'ERROR: inconsistency in boxfile and s_mpirun'
			
		endif
		
		write(*,*) 'number of processors:', pp
		read(15,*) junk
		
		do k=1,p,1
			
			read(15,*) dummy
			read(15,*) ix_multi_loc, iy_multi_loc, iz_multi_loc
			read(15,*) nx_min_loc, nx_max_loc
			read(15,*) ny_min_loc, ny_max_loc
			read(15,*) nz_min_loc, nz_max_loc
			read(15,*) xmin_loc, xmax_loc
			read(15,*) ymin_loc, ymax_loc
			read(15,*) zmin_loc, zmax_loc
			read(15,*) junk
			
			if (k==1) then
			
				ix_multi=ix_multi_loc
				iy_multi=iy_multi_loc
				iz_multi=iz_multi_loc
				
				nx=nx_max_loc-nx_min_loc
				ny=ny_max_loc-ny_min_loc
				nz=nz_max_loc-nz_min_loc
				
				xmin=xmin_loc
				xmax=xmax_loc
				ymin=ymin_loc
				ymax=ymax_loc
				zmin=zmin_loc
				zmax=zmax_loc
				
				write(99,*) '----------------------------------'
				write(99,*) '- geometrical parameters ---------'
				write(99,*) '----------------------------------'
				write(99,*) 'itheta_multi=',ix_multi,' iphi_multi=',iy_multi,' iz_multi=',iz_multi
				write(99,*) 'n_theta=',nx,' n_phi=',ny,' n_z=',nz
				write(99,*) 'theta_min=',xmin*180/pi,' theta_max=',xmax*180/pi
				write(99,*) 'phi_min=',ymin*180/pi,' phi_max=',ymax*180/pi
				write(99,*) 'z_min=',zmin,' z_max=',zmax
				write(99,*) '----------------------------------'
				
			else

				call MPI_Send(ix_multi_loc, 1, MPI_INTEGER, k-1, 1, MPI_COMM_WORLD, ierr)
				call MPI_Send(iy_multi_loc, 1, MPI_INTEGER, k-1, 2, MPI_COMM_WORLD, ierr)
				call MPI_Send(iz_multi_loc, 1, MPI_INTEGER, k-1, 3, MPI_COMM_WORLD, ierr)
				call MPI_Send(nx_max_loc-nx_min_loc, 1, MPI_INTEGER, k-1, 4, MPI_COMM_WORLD, ierr)
				call MPI_Send(ny_max_loc-ny_min_loc, 1, MPI_INTEGER, k-1, 5, MPI_COMM_WORLD, ierr)
				call MPI_Send(nz_max_loc-nz_min_loc, 1, MPI_INTEGER, k-1, 6, MPI_COMM_WORLD, ierr)
				call MPI_Send(xmin_loc, 1, MPI_REAL, k-1, 7, MPI_COMM_WORLD, ierr)
				call MPI_Send(xmax_loc, 1, MPI_REAL, k-1, 8, MPI_COMM_WORLD, ierr)
				call MPI_Send(ymin_loc, 1, MPI_REAL, k-1, 9, MPI_COMM_WORLD, ierr)
				call MPI_Send(ymax_loc, 1, MPI_REAL, k-1, 10, MPI_COMM_WORLD, ierr)
				call MPI_Send(zmin_loc, 1, MPI_REAL, k-1, 11, MPI_COMM_WORLD, ierr)
				call MPI_Send(zmax_loc, 1, MPI_REAL, k-1, 12, MPI_COMM_WORLD, ierr)
				
			endif
			
		enddo
		
		close(UNIT=15)
		
	else
		
		call MPI_Recv(ix_multi, 1, MPI_INTEGER, 0, 1, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(iy_multi, 1, MPI_INTEGER, 0, 2, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(iz_multi, 1, MPI_INTEGER, 0, 3, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(nx, 1, MPI_INTEGER, 0, 4, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(ny, 1, MPI_INTEGER, 0, 5, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(nz, 1, MPI_INTEGER, 0, 6, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(xmin, 1, MPI_REAL, 0, 7, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(xmax, 1, MPI_REAL, 0, 8, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(ymin, 1, MPI_REAL, 0, 9, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(ymax, 1, MPI_REAL, 0, 10, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(zmin, 1, MPI_REAL, 0, 11, MPI_COMM_WORLD, status, ierr)
		call MPI_Recv(zmax, 1, MPI_REAL, 0, 12, MPI_COMM_WORLD, status, ierr)
		
		write(99,*) '----------------------------------'
		write(99,*) '- geometrical parameters ---------'
		write(99,*) '----------------------------------'
		write(99,*) 'itheta_multi=',ix_multi,' iphi_multi=',iy_multi,' iz_multi=',iz_multi
		write(99,*) 'n_theta=',nx,' n_phi=',ny,' n_z=',nz
		write(99,*) 'theta_min=',xmin*180/pi,' theta_max=',xmax*180/pi
		write(99,*) 'phi_min=',ymin*180/pi,' phi_max=',ymax*180/pi
		write(99,*) 'z_min=',zmin,' z_max=',zmax
		write(99,*) '----------------------------------'
		
	endif

	call mpi_bcast(p,1,mpi_integer,0,mpi_comm_world,ierr)
	call mpi_bcast(px,1,mpi_integer,0,mpi_comm_world,ierr)
	call mpi_bcast(py,1,mpi_integer,0,mpi_comm_world,ierr)
	call mpi_bcast(pz,1,mpi_integer,0,mpi_comm_world,ierr)
	
	write(99,*) '----------------------------------'
	write(99,*) '- parallelisation parameters -----'
	write(99,*) '----------------------------------'
	write(99,*) 'total number of processors:', p
	write(99,*) 'number of processors in theta direction:', px
	write(99,*) 'number of processors in phi direction:', py
	write(99,*) 'number of processors in z direction:', pz
	write(99,*) '----------------------------------'
	
	
	!=====================================================================
	! read Par file, only process 0
	!=====================================================================

	if (my_rank==0) then

		write(*,*) 'process 0 reading Par file'
		
		write(99,*) '----------------------------------'
		write(99,*) '- Par file -----------------------'
		write(99,*) '----------------------------------'
		
		OPEN (UNIT=15,FILE='Par',STATUS='OLD',ACTION='READ') 

		READ(15,*)junk
		READ(15,*)nt
		WRITE(99,*) 'number of time steps: nt=', nt
		READ(15,*)dt
		WRITE(99,*) 'time increment: dt=', dt
		READ(15,*)junk
		READ(15,*)xxs
		WRITE(99,*) 'theta-coordinate of the source: xxs=',xxs
		READ(15,*)yys
		WRITE(99,*) 'phi-coordinate of the source: yys=',yys
		READ(15,*)zzs
		WRITE(99,*) 'z-coordinate of the source (depth): zzs=',zzs
		READ(15,*)source_type
		WRITE(99,*) 'source_type: ',source_type

                READ(15,*)MOM_xx
                WRITE(99,*) 'Mxx: ', MOM_xx
                READ(15,*)MOM_yy
                WRITE(99,*) 'Myy: ', MOM_yy
                READ(15,*)MOM_zz
                WRITE(99,*) 'Mzz: ', MOM_zz
                READ(15,*)MOM_xy
                WRITE(99,*) 'Mxy: ', MOM_xy
                READ(15,*)MOM_xz
                WRITE(99,*) 'Mxz: ', MOM_xz
                READ(15,*) MOM_yz
                WRITE(99,*) 'Myz: ', MOM_yz
                
		READ(15,*)junk
		
                READ(15,*)ssamp
		WRITE(99,*) 'output rate: ssamp=',ssamp
		READ(15,*)iplanex
		WRITE(99,*) 'x-output plane: iplanex=',iplanex
		READ(15,*)iplaney
		WRITE(99,*) 'y-output plane: iplaney=',iplaney
		READ(15,*)iplanez
		WRITE(99,*) 'z-output plane: iplanez=',iplanez
		READ(15,*)xmin_global
		WRITE(99,*) 'global minimum theta-extension: xmin=',xmin_global
		READ(15,*)xmax_global
		WRITE(99,*) 'global maximum theta-extension: xmax=',xmax_global
		READ(15,*)ymin_global
		WRITE(99,*) 'global minimum phi-extension: ymin=',ymin_global
		READ(15,*)ymax_global
		WRITE(99,*) 'global maximum phi-extension: ymax=',ymax_global
		READ(15,*)zmin_global
		WRITE(99,*) 'global minimum z-extension: zmin=',zmin_global
		READ(15,*)zmax_global
		WRITE(99,*) 'global maximum z-extension: zmax=',zmax_global
		READ(15,*)is_aniso
		WRITE(99,*) 'anisotropy on: ', is_aniso
		READ(15,*)is_diss
		WRITE(99,*) 'dissipation on: ', is_diss
		WRITE(*,*) 'dissipation on: ', is_diss
		READ(15,*) is_formatted
		WRITE(99,*) 'formatted output: ', is_formatted
		READ(15,*)junk
		READ(15,'(A100)')ofd
		WRITE(99,*) 'output field directory: ', ofd

		write(99,*) '----------------------------------'
		
		CLOSE(15)

                xxs=xxs*pi/180
                yys=yys*pi/180

                xmin_global=xmin_global*pi/180
                xmax_global=xmax_global*pi/180
                ymin_global=ymin_global*pi/180
                ymax_global=ymax_global*pi/180
                
	endif
	
	!=====================================================================
	! broadcast input parameters to the other processes
	!=====================================================================
	
	call mpi_bcast(nt, 1, mpi_integer, 0, mpi_comm_world, ierr)
	call mpi_bcast(dt, 1, mpi_real, 0, mpi_comm_world, ierr)

	call mpi_bcast(xxs, 1, mpi_real, 0, mpi_comm_world, ierr)
	call mpi_bcast(yys, 1, mpi_real, 0, mpi_comm_world, ierr)
	call mpi_bcast(zzs, 1, mpi_real, 0, mpi_comm_world, ierr)
	call mpi_bcast(source_type, 1, mpi_integer, 0, mpi_comm_world, ierr)

        call mpi_bcast(MOM_xx, 1, mpi_real, 0, mpi_comm_world, ierr)
        call mpi_bcast(MOM_yy, 1, mpi_real, 0, mpi_comm_world, ierr)
        call mpi_bcast(MOM_zz, 1, mpi_real, 0, mpi_comm_world, ierr)
        call mpi_bcast(MOM_xy, 1, mpi_real, 0, mpi_comm_world, ierr)
        call mpi_bcast(MOM_xz, 1, mpi_real, 0, mpi_comm_world, ierr)
        call mpi_bcast(MOM_yz, 1, mpi_real, 0, mpi_comm_world, ierr)

	call mpi_bcast(ssamp, 1, mpi_integer, 0, mpi_comm_world, ierr)
	call mpi_bcast(iplanex, 1, mpi_integer, 0, mpi_comm_world, ierr)
	call mpi_bcast(iplaney, 1, mpi_integer, 0, mpi_comm_world, ierr)
	call mpi_bcast(iplanez, 1, mpi_integer, 0, mpi_comm_world, ierr)
	
	call mpi_bcast(xmin_global, 1, mpi_real, 0, mpi_comm_world, ierr)
	call mpi_bcast(xmax_global, 1, mpi_real, 0, mpi_comm_world, ierr)
	call mpi_bcast(ymin_global, 1, mpi_real, 0, mpi_comm_world, ierr)
	call mpi_bcast(ymax_global, 1, mpi_real, 0, mpi_comm_world, ierr)
	call mpi_bcast(zmin_global, 1, mpi_real, 0, mpi_comm_world, ierr)
	call mpi_bcast(zmax_global, 1, mpi_real, 0, mpi_comm_world, ierr)
	
	call mpi_bcast(is_aniso, 1, mpi_integer, 0, mpi_comm_world, ierr)
	call mpi_bcast(is_diss, 1, mpi_integer, 0, mpi_comm_world, ierr)
	call mpi_bcast(is_formatted, 1, mpi_integer, 0, mpi_comm_world, ierr)
	
	call mpi_bcast(ofd,100,mpi_character,0,mpi_comm_world,ierr)
	
	!=====================================================================
        ! read Par_ad file, only process 0, and broadcast to others
        !=====================================================================

        if (my_rank==0) then

           write(*,*) 'process 0 reading Par_ad file'

           write(99,*) '----------------------------------'
	   write(99,*) '- Par_ad file --------------------'
	   write(99,*) '----------------------------------'
		
	   open(unit=15,file='Par_ad',status='old',action='read') 

           read(15,*) junk
           read(15,*) samp_ad
           write(99,*) 'forward field storing rate: ', samp_ad
           read(15,'(A100)') ffd
	   write(99,*) 'forward field directory: ', ffd
	   
           close(unit=15)

        endif

	call mpi_bcast(samp_ad, 1, mpi_integer, 0, mpi_comm_world, ierr)
	call mpi_bcast(ffd,100,mpi_character,0,mpi_comm_world,ierr)
		

	!=============================================================================
	! read source time function, adequate process only
	!=============================================================================

	if ((xmin .LE. xxs) .AND. (xmax .GE. xxs) .AND. (ymin .LE. yys) .AND. (ymax .GE. yys) .AND. &
		(zmin .LE. zmax_global-zzs) .AND. (zmax .GE. zmax_global-zzs)) then
		
		write(99,*) 'process ', my_rank, 'reading source time function'
                write(*,*) 'process', my_rank, 'speaking ...'
		is_source=1.0
		
		open(UNIT=15,FILE='../MODELS/stf',STATUS='OLD',ACTION='READ')
	
		do i=1, nt, 1
			read(15,*) so(i)
                        write(99,*) so(i)
		end do
	
		close(UNIT=15)
    
	else
		
		write(99,*) 'source not located in this processor box'
		is_source=0.0
		
	endif
	
	call mpi_reduce(is_source*my_rank,source_processor,1,mpi_real,mpi_sum,0,mpi_comm_world,ierr)
	
	if (my_rank==0) then
		
                call int2str(int(source_processor),junk)
                
		write(*,*) 'process', junk, 'reading source time function'
                write(99,*) 'process', source_processor, 'reading source time function'
	
	endif

	!=============================================================================
	! Read structural information (rho, mu, lambda, A, B, C), formatted or unformatted
	!=============================================================================

	call int2str(my_rank,junk)

	if (my_rank==0) then
		write(*,*) "reading structural information"
	endif
	
	write(99,*) '----------------------------------'
	write(99,*) '- structural input ---------------'
	write(99,*) '----------------------------------'
		
	if (is_diss==1) then
		write(99,*) 'Dissipation on'
	else
		write(99,*) 'Dissipation off'
	endif
	
	if (is_aniso==1) then
		write(99,*) 'Anisotropy on'
	else
		write(99,*) 'Anisotropy off'
	endif

	if (is_aniso==1) then
				
		open (UNIT=21,FILE='../MODELS/A'//junk(1:len_trim(junk)),STATUS='OLD',ACTION='READ',form='unformatted')
		open (UNIT=22,FILE='../MODELS/B'//junk(1:len_trim(junk)),STATUS='OLD',ACTION='READ',form='unformatted')
		open (UNIT=23,FILE='../MODELS/C'//junk(1:len_trim(junk)),STATUS='OLD',ACTION='READ',form='unformatted')
			
		read(21) A(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)
		read(22) B(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)
		read(23) C(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)

		close(unit=21)
		close(unit=22)
		close(unit=23)
		
	endif
	
	if (is_diss==1) then

                tau(:,:,:,:,:,:)=0.180
                tau_sp(1,:,:,:,:,:,:)=8.59
                tau_sp(2,:,:,:,:,:,:)=1.00
			
	endif

		
	open (UNIT=15,FILE='../MODELS/rhoinv'//junk(1:len_trim(junk)),STATUS='OLD',ACTION='READ',form='unformatted')
	open (UNIT=16,FILE='../MODELS/mu'//junk(1:len_trim(junk)),STATUS='OLD',ACTION='READ',form='unformatted')
	open (UNIT=17,FILE='../MODELS/lambda'//junk(1:len_trim(junk)),STATUS='OLD',ACTION='READ',form='unformatted')
			
	read(15) rhoinv(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)
	read(16) mu(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)
	read(17) lambda(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)
				
	close(unit=15)
	close(unit=16)
	close(unit=17)
	

	write(99,*) 'minimum rho: ',minval(1/rhoinv(0:nx,0:ny,0:nz,:,:,:)), &
		' maximum rho: ',maxval(1/rhoinv(0:nx,0:ny,0:nz,:,:,:))
	write(99,*) 'minimum lambda: ',minval(lambda(0:nx,0:ny,0:nz,:,:,:)), &
		' maximum lambda: ',maxval(lambda(0:nx,0:ny,0:nz,:,:,:))
	write(99,*) 'minimum mu: ',minval(mu(0:nx,0:ny,0:nz,:,:,:)), &
		' maximum mu: ', maxval(mu(0:nx,0:ny,0:nz,:,:,:))
	
	if (is_aniso==1) then
		
		write(99,*) 'minimum A: ',minval(A(0:nx,0:ny,0:nz,:,:,:)), ' maximum A: ', maxval(A(0:nx,0:ny,0:nz,:,:,:))
		write(99,*) 'minimum B: ',minval(B(0:nx,0:ny,0:nz,:,:,:)), ' maximum B: ', maxval(B(0:nx,0:ny,0:nz,:,:,:))
		write(99,*) 'minimum C: ',minval(C(0:nx,0:ny,0:nz,:,:,:)), ' maximum C: ', maxval(C(0:nx,0:ny,0:nz,:,:,:))
		
	endif
	
	if (is_diss==1) then
		
		write(99,*) 'minimum tau: ',minval(tau(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)),&
			' maximum tau: ', maxval(tau(0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd))
  
               write(99,*) 'minima tau_sp: '

               do k=1,nrdiss
                  write(99,*) minval(tau_sp(k,0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)), ', '
               enddo

               write(99,*) 'maxima tau_sp: '

               do k=1,nrdiss
                  write(99,*) maxval(tau_sp(k,0:nx,0:ny,0:nz,0:lpd,0:lpd,0:lpd)), ', '
               enddo
		
	endif
		
	write(99,*) '----------------------------------'
	
	!======================================================================
	! read receiver locations, process  0 only
	!======================================================================
	
	if (my_rank==0) then
		
		write(*,*) 'process 0 reading receiver locations'
	
		write(99,*) '----------------------------------'
		write(99,*) '- receiver locations -------------'
                write(99,*) '- theta, phi, depth in m ---------'
		write(99,*) '----------------------------------'	
		
		open(unit=10, file='recfile')
		
		read(10,*) nr_global	!number of receivers
		
		do i=1,nr_global

			read(10,*) station_name_global(i) 
			read(10,*) recloc_global(1,i), recloc_global(2,i), recloc_global(3,i)
			write(99,*) station_name_global(i), recloc_global(1,i), recloc_global(2,i), recloc_global(3,i)

                        recloc_global(1,i)=recloc_global(1,i)*pi/180
                        recloc_global(2,i)=recloc_global(2,i)*pi/180

		enddo
		
		close(10)
		
		write(99,*) '----------------------------------'
		
	endif
	
	call mpi_bcast(station_name_global, 4*maxnr, mpi_character, 0, mpi_comm_world, ierr)
	call mpi_bcast(recloc_global, 3*maxnr, mpi_real, 0, mpi_comm_world, ierr)
	call mpi_bcast(nr_global,1,mpi_integer,0,mpi_comm_world,ierr)

	write(99,*) 'end input'
	write(99,*) '----------------------------------------'
        
end subroutine ses3d_input


!==============================================================================
! subroutine for integer to string conversion
!==============================================================================

subroutine int2str(value, string)
implicit none

	integer, intent(in) :: value
	character(len=*), intent(inout) :: string
	
	character(len=10) :: c
	integer :: k, n, new_value, is
	real :: e
	
	e=1e9
	is=0
	
	if (value==0) then
		string(1:1)='0'
		string(2:10)=' '
	else
	
		new_value=value
	
		do k=1,10
			c(k:k)=char(floor(new_value/e)+48)
		
			if ((floor(new_value/e)==0) .and. (is==0)) then
				n=k
			else
				is=1
			endif
		
			new_value=new_value-e*floor(new_value/e)
			e=e/10
			string(k:k)=' '
		
		enddo
		
		string(1:10-n)=c(n+1:10)
		
	endif
	
	if (len(string)>10) then
		string(11:len(string))=' '
	endif

return
end subroutine int2str


