!==============================================================================
! store field values at lower order collocation points
!==============================================================================

subroutine ses3d_store(field,name,n,deg,tag)
use parameters
use variables
implicit none
include 'mpif.h'

        ! field - field variable
        ! name  - string containing the name of the field variable
        ! n     - iteration number
        ! deg   - polynomial degree of the stored field

        !======================================================================
        ! local variables
        !======================================================================

        integer, intent(in) :: n, deg, tag
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd), intent(in) :: field
        character(len=*), intent(in) :: name

        real :: cknots(0:7), lx, ly, lz

        real, dimension(0:nx,0:ny,0:nz,0:deg,0:deg,0:deg) :: cfield

        integer :: i, j, k, ip, iq, ir

        character(len=10) :: dummy, ns
        character(len=50) :: fn

        !======================================================================
        ! compress only if deg<lpd
        !======================================================================

        if (deg<lpd) then
        	
           !======================================================================
           ! determine collocation points (knots) and integration weights
           !======================================================================
	
           if (deg==2) then
		
		cknots(0)=-1.0
		cknots(1)=0.0
		cknots(2)=1.0
		
           elseif (deg==3) then
		
		cknots(0)=-1.0
		cknots(1)=-0.4472135954999579
		cknots(2)=0.4472135954999579
		cknots(3)=1.0
		
           elseif (deg==4) then
		
		cknots(0)=-1.0
		cknots(1)=-0.6546536707079772
		cknots(2)=0.0
		cknots(3)=0.6546536707079772
		cknots(4)=1.0
		
           elseif (deg==5) then
		
		cknots(0)=-1.0
		cknots(1)=-0.7650553239294647
		cknots(2)=-0.2852315164806451
		cknots(3)=0.2852315164806451
		cknots(4)=0.7650553239294647
		cknots(5)=1.0
		
           elseif (deg==6) then
		
		cknots(0)=-1.0
		cknots(1)=-0.8302238962785670
		cknots(2)=-0.4688487934707142
		cknots(3)=0.0
		cknots(4)=0.4688487934707142
		cknots(5)=0.8302238962785670
		cknots(6)=1.0
		
           elseif (deg==7) then
		
		cknots(0)=-1.0
		cknots(1)=-0.8717401485096066
		cknots(2)=-0.5917001814331423
		cknots(3)=-0.2092992179024789
		cknots(4)=0.2092992179024789
		cknots(5)=0.5917001814331423
		cknots(6)=0.8717401485096066
		cknots(7)=1.0
		
           endif

           !======================================================================
           ! generate lower degree interpolant
           !======================================================================

           cfield(:,:,:,:,:,:)=0.0

           do i=0,deg
           do j=0,deg
           do k=0,deg

              do ip=0,lpd
              
                 call lgll(lpd,ip,cknots(i),lx)

                 do iq=0,lpd

                    call lgll(lpd,iq,cknots(j),ly)

                    do ir=0,lpd

                       call lgll(lpd,ir,cknots(k),lz) 

                       cfield(0:nx,0:ny,0:nz,i,j,k)=cfield(0:nx,0:ny,0:nz,i,j,k)+field(0:nx,0:ny,0:nz,ip,iq,ir)*lx*ly*lz

                    enddo
                 enddo
              enddo

           enddo
           enddo
           enddo

        endif

        !======================================================================
        ! store field variable
        !======================================================================

        call int2str(my_rank,dummy)

        if (n==1) then

           !fn='/scratch/fichtner/SI002/'//name(1:len_trim(name))//'_'//dummy(1:len_trim(dummy))
           !fn='DATA/OUTPUT/'//name(1:len_trim(name))//'_'//dummy(1:len_trim(dummy))
	   fn=ffd(1:len_trim(ffd))//name(1:len_trim(name))//'_'//dummy(1:len_trim(dummy))

           write(99,*) fn

           open(unit=tag,file=fn,action='write',form='unformatted')

        endif

        if (deg<lpd) then

           write(tag) cfield

        else

           write(tag) field

        endif

        if (n==nt) then

           close(unit=tag)

        endif

end subroutine ses3d_store


!==============================================================================
! read field values at lower order collocation points and interpolate
! to higher order collocation points
!==============================================================================

subroutine ses3d_restore(field,name,idx,deg,tag)
use parameters
use variables
implicit none
include 'mpif.h'

   
	!======================================================================
	! local variables
	!======================================================================
	
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd), intent(inout) :: field
        integer, intent(in) :: idx, deg, tag
        character(len=*), intent(in) :: name

        integer :: i, j, k, il, im, in
        
        real :: lx, ly, lz
        real, dimension(0:nx,0:ny,0:nz,0:deg,0:deg,0:deg) :: cfield

        character(len=10) :: dummy
        character(len=50) :: fn

	!======================================================================
	! read lower degree polynomial collocation point values
	!======================================================================

        call int2str(my_rank,dummy)

        !- open file in first iteration

        if (idx==1) then

           !fn='/scratch/fichtner/SI002/'//name(1:len_trim(name))//'_'//dummy(1:len_trim(dummy))
	   fn=ffd(1:len_trim(ffd))//name(1:len_trim(name))//'_'//dummy(1:len_trim(dummy))

           open(unit=tag,file=fn,action='read',form='unformatted',position='append')
           backspace(unit=tag)

        endif

        !- read and check if interpolation is necessary

        if (deg<lpd) then

           read(tag) cfield
           backspace(unit=tag)
           backspace(unit=tag)

        else

           read(tag) field
           backspace(unit=tag)
           backspace(unit=tag)

        endif

        !- close file in last iteration

        if (idx==nt) then

           close(unit=tag)
           backspace(unit=tag)
           backspace(unit=tag)

        endif

	!======================================================================
	! interpolate to higher polynomial collocation point values if necessary
	!======================================================================

        if (deg<lpd) then

           field=0.0

           do i=0,lpd
           do j=0,lpd
           do k=0,lpd

              do il=0,deg
              
                 call lgll(deg,il,knots(i),lx)

                 do im=0,deg

                    call lgll(deg,im,knots(j),ly)

                    do in=0,deg

                       call lgll(deg,in,knots(k),lz)

                       field(0:nx,0:ny,0:nz,i,j,k)=field(0:nx,0:ny,0:nz,i,j,k)+cfield(0:nx,0:ny,0:nz,il,im,in)*lx*ly*lz

                    enddo

                 enddo

              enddo

         enddo
         enddo
         enddo

      endif

end subroutine ses3d_restore


subroutine ses3d_output(n)
use parameters
use variables
implicit none
include 'mpif.h'

	!======================================================================
	! local variables
	!======================================================================
	
	integer, intent(in) :: n
        integer :: i, j 
	character(len=10) :: dummy, ns
	character(len=50) :: fn
	character(len=6) :: sname
	
	integer :: k,lk

        real :: rec_x, rec_y, rec_z

	!======================================================================
	! write displacement fields to files
	!======================================================================
	
	call int2str(my_rank,dummy)
	call int2str(n,ns)
	
	fn=ofd(1:len_trim(ofd))//'OUTPUT/ux_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
	open(unit=10,file=fn,action='write',form='unformatted')
        fn=ofd(1:len_trim(ofd))//'OUTPUT/uy_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
	open(unit=11,file=fn,action='write',form='unformatted')
	fn=ofd(1:len_trim(ofd))//'OUTPUT/uz_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
      	open(unit=12,file=fn,action='write',form='unformatted')
	
	write(10) ux
	write(11) uy
	write(12) uz
	
	close(unit=10)
	close(unit=11)
	close(unit=12)

! PRECONDITIONER
!
!	fn=ofd(1:len_trim(ofd))//'preco_fw_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
!        open(unit=13,file=fn,action='write',form='unformatted')
!
!        write(13) preco
!
!        close(unit=13)

        !======================================================================
        ! write saving vector to files
        !======================================================================

! following lines commented out by heiner

!        open(unit=10,file=ffd(1:len_trim(ffd))//'saving_vector_'//dummy(1:len_trim(dummy)),action='write')
!
!        do k=1,nt
!
!           write(10,*) saving_vector(k)
!
!        enddo
!
!        close(unit=10)


	!======================================================================
	! write seismograms to files
	!======================================================================

	if (nr>0) then

	do i=1,nr

		sname=station_name_local(i)
	
		open(unit=51,file=ofd(1:len_trim(ofd))//'OUTPUT/'//sname(1:len_trim(sname))//'_x',action='WRITE')
		open(unit=52,file=ofd(1:len_trim(ofd))//'OUTPUT/'//sname(1:len_trim(sname))//'_y',action='WRITE')
		open(unit=53,file=ofd(1:len_trim(ofd))//'OUTPUT/'//sname(1:len_trim(sname))//'_z',action='WRITE')	
	
		write(51,*) 'theta component seismograms'
		write(52,*) 'phi component seismograms'
		write(53,*) 'r component seismograms'
	
		write(51,*) 'nr=', nr
		write(51,*) 'nt=', nt
		write(51,*) 'dt=', dt
	
		write(52,*) 'nr=', nr
		write(52,*) 'nt=', nt
		write(52,*) 'dt=', dt
	
		write(53,*) 'nr=', nr
		write(53,*) 'nt=', nt
		write(53,*) 'dt=', dt

		write(51,*) 'x=', recloc(1,i), 'y=', recloc(2,i), 'z=', recloc(3,i)
                write(52,*) 'x=', recloc(1,i), 'y=', recloc(2,i), 'z=', recloc(3,i)
		write(53,*) 'x=', recloc(1,i), 'y=', recloc(2,i), 'z=', recloc(3,i)

		do j=1,nt
		
			write(51,*) seismogram_x(i,j)
			write(52,*) seismogram_y(i,j)
			write(53,*) seismogram_z(i,j)
			
		enddo

		close(unit=51)
		close(unit=52)
		close(unit=53)

	enddo

	
		!open(unit=51,file='DATA/OUTPUT/seismogram_x'//dummy(1:len_trim(dummy)),action='WRITE')
		!open(unit=52,file='DATA/OUTPUT/seismogram_y'//dummy(1:len_trim(dummy)),action='WRITE')
		!open(unit=53,file='DATA/OUTPUT/seismogram_z'//dummy(1:len_trim(dummy)),action='WRITE')
	
!		open(unit=51,file=ofd(1:len_trim(ofd))//'seismogram_x'//dummy(1:len_trim(dummy)),action='WRITE')
!		open(unit=52,file=ofd(1:len_trim(ofd))//'seismogram_y'//dummy(1:len_trim(dummy)),action='WRITE')
!		open(unit=53,file=ofd(1:len_trim(ofd))//'seismogram_z'//dummy(1:len_trim(dummy)),action='WRITE')
		
!		write(51,*) 'theta component seismograms'
!		write(52,*) 'phi component seismograms'
!		write(53,*) 'r component seismograms'
	
!		write(51,*) 'nr=', nr
!		write(51,*) 'nt=', nt
!		write(51,*) 'dt=', dt
	
!		write(52,*) 'nr=', nr
!		write(52,*) 'nt=', nt
!		write(52,*) 'dt=', dt
	
!		write(53,*) 'nr=', nr
!		write(53,*) 'nt=', nt
!		write(53,*) 'dt=', dt
	
!		do i=1,nr
     
!			write(51,*) 'x=', recloc(1,i), 'y=', recloc(2,i), 'z=', recloc(3,i)
!                       write(52,*) 'x=', recloc(1,i), 'y=', recloc(2,i), 'z=', recloc(3,i)
!			write(53,*) 'x=', recloc(1,i), 'y=', recloc(2,i), 'z=', recloc(3,i)
		
!			do j=1,nt
		
!				write(51,*) seismogram_x(i,j)
!				write(52,*) seismogram_y(i,j)
!				write(53,*) seismogram_z(i,j)
			
!			enddo
		
!		enddo
	
!		close(unit=51)
!		close(unit=52)
!		close(unit=53)
		
	endif
 
end subroutine ses3d_output


subroutine record_seismograms
use parameters
use variables
implicit none
include 'mpif.h'

	!======================================================================
	! local variables
	!======================================================================

        integer :: idx
        integer :: i, j, k
        real :: lx, ly, lz
        real :: dummy

        !======================================================================
        ! interpolate displacement field
        !======================================================================

        if (nr>0) then

           do idx=1,nr

              do i=0,lpd

                 call lgll(lpd,i,recloc_std(1,idx),lx)
                 do j=0,lpd

                    call lgll(lpd,j,recloc_std(2,idx),ly)
                    do k=0,lpd

                       call lgll(lpd,k,recloc_std(3,idx),lz)

                       dummy=lx*ly*lz

                       seismogram_x(idx,it)=seismogram_x(idx,it)+ux(rx(idx),ry(idx),rz(idx),i,j,k)*dummy
                       seismogram_y(idx,it)=seismogram_y(idx,it)+uy(rx(idx),ry(idx),rz(idx),i,j,k)*dummy
                       seismogram_z(idx,it)=seismogram_z(idx,it)+uz(rx(idx),ry(idx),rz(idx),i,j,k)*dummy

                    enddo
                 enddo
              enddo

           enddo

        endif ! (nr>0)


end subroutine record_seismograms
