!==============================================================================
! store field values at lower order collocation points
!==============================================================================

subroutine ses3d_store(field,name,n,deg,tag)
use parameters
use variables
implicit none
include 'mpif.h'

        ! field - field variable
        ! name  - string containing the name of the field variable
        ! n     - iteration number
        ! deg   - polynomial degree of the stored field

        !======================================================================
        ! local variables
        !======================================================================

        integer, intent(in) :: n, deg, tag
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd), intent(in) :: field
        character(len=*), intent(in) :: name

        real :: cknots(0:7), lx, ly, lz

        real, dimension(0:nx,0:ny,0:nz,0:deg,0:deg,0:deg) :: cfield

        integer :: i, j, k, ip, iq, ir

        character(len=10) :: dummy, ns
        character(len=50) :: fn

        !======================================================================
        ! compress only if deg<lpd
        !======================================================================

        if (deg<lpd) then
        	
           !======================================================================
           ! determine collocation points (knots) and integration weights
           !======================================================================
	
           if (deg==2) then
		
		cknots(0)=-1.0
		cknots(1)=0.0
		cknots(2)=1.0
		
           elseif (deg==3) then
		
		cknots(0)=-1.0
		cknots(1)=-0.4472135954999579
		cknots(2)=0.4472135954999579
		cknots(3)=1.0
		
           elseif (deg==4) then
		
		cknots(0)=-1.0
		cknots(1)=-0.6546536707079772
		cknots(2)=0.0
		cknots(3)=0.6546536707079772
		cknots(4)=1.0
		
           elseif (deg==5) then
		
		cknots(0)=-1.0
		cknots(1)=-0.7650553239294647
		cknots(2)=-0.2852315164806451
		cknots(3)=0.2852315164806451
		cknots(4)=0.7650553239294647
		cknots(5)=1.0
		
           elseif (deg==6) then
		
		cknots(0)=-1.0
		cknots(1)=-0.8302238962785670
		cknots(2)=-0.4688487934707142
		cknots(3)=0.0
		cknots(4)=0.4688487934707142
		cknots(5)=0.8302238962785670
		cknots(6)=1.0
		
           elseif (deg==7) then
		
		cknots(0)=-1.0
		cknots(1)=-0.8717401485096066
		cknots(2)=-0.5917001814331423
		cknots(3)=-0.2092992179024789
		cknots(4)=0.2092992179024789
		cknots(5)=0.5917001814331423
		cknots(6)=0.8717401485096066
		cknots(7)=1.0
		
           endif

           !======================================================================
           ! generate lower degree interpolant
           !======================================================================

           cfield(:,:,:,:,:,:)=0.0

           do i=0,deg
           do j=0,deg
           do k=0,deg

              do ip=0,lpd
              
                 call lgll(lpd,ip,cknots(i),lx)

                 do iq=0,lpd

                    call lgll(lpd,iq,cknots(j),ly)

                    do ir=0,lpd

                       call lgll(lpd,ir,cknots(k),lz) 

                       cfield(0:nx,0:ny,0:nz,i,j,k)=cfield(0:nx,0:ny,0:nz,i,j,k)+field(0:nx,0:ny,0:nz,ip,iq,ir)*lx*ly*lz

                    enddo
                 enddo
              enddo

           enddo
           enddo
           enddo

        endif

        !======================================================================
        ! store field variable
        !======================================================================

        call int2str(my_rank,dummy)

        if (n==1) then

           !fn='/scratch/fichtner/SI002/'//name(1:len_trim(name))//'_'//dummy(1:len_trim(dummy))
	   fn=ffd(1:len_trim(ffd))//name(1:len_trim(name))//'_'//dummy(1:len_trim(dummy))

           write(99,*) fn

           open(unit=tag,file=fn,action='write',form='unformatted')

        endif

        if (deg<lpd) then

           write(tag) cfield

        else

           write(tag) field

        endif

        if (n==nt) then

           close(unit=tag)

        endif

end subroutine ses3d_store


!==============================================================================
! read field values at lower order collocation points and interpolate
! to higher order collocation points
!==============================================================================

subroutine ses3d_restore(field,name,idx,deg,tag)
use parameters
use variables
implicit none
include 'mpif.h'

   
	!======================================================================
	! local variables
	!======================================================================
	
        real, dimension(0:nx_max,0:ny_max,0:nz_max,0:lpd,0:lpd,0:lpd), intent(inout) :: field
        integer, intent(in) :: idx, deg, tag
        character(len=*), intent(in) :: name

        integer :: i, j, k, il, im, in
        
        real :: lx, ly, lz
        real, dimension(0:nx,0:ny,0:nz,0:deg,0:deg,0:deg) :: cfield

        character(len=10) :: dummy
        character(len=50) :: fn

	!======================================================================
	! read lower degree polynomial collocation point values
	!======================================================================

        call int2str(my_rank,dummy)

        !- open file in first iteration

        if (idx==1) then

           !fn='/scratch/fichtner/SI002/'//name(1:len_trim(name))//'_'//dummy(1:len_trim(dummy))
	   fn=ffd(1:len_trim(ffd))//name(1:len_trim(name))//'_'//dummy(1:len_trim(dummy))

           open(unit=tag,file=fn,action='read',form='unformatted',position='append')
           backspace(unit=tag)

        endif

        !- read and check if interpolation is necessary

        if (deg<lpd) then

           read(tag) cfield
           backspace(unit=tag)
           backspace(unit=tag)

        else

           read(tag) field
           backspace(unit=tag)
           backspace(unit=tag)

        endif

        !- close file in last iteration

        if (idx==nt) then

           close(unit=tag)

        endif

	!======================================================================
	! interpolate to higher polynomial collocation point values if necessary
	!======================================================================

        if (deg<lpd) then

           field=0.0

           do i=0,lpd
           do j=0,lpd
           do k=0,lpd

              do il=0,deg
              
                 call lgll(deg,il,knots(i),lx)

                 do im=0,deg

                    call lgll(deg,im,knots(j),ly)

                    do in=0,deg

                       call lgll(deg,in,knots(k),lz)

                       field(0:nx,0:ny,0:nz,i,j,k)=field(0:nx,0:ny,0:nz,i,j,k)+cfield(0:nx,0:ny,0:nz,il,im,in)*lx*ly*lz

                    enddo

                 enddo

              enddo

         enddo
         enddo
         enddo

      endif

end subroutine ses3d_restore


subroutine ses3d_output(n)
use parameters
use variables
implicit none
include 'mpif.h'

	!======================================================================
	! local variables
	!======================================================================
	
	integer, intent(in) :: n
        integer :: i, j 
	character(len=10) :: dummy, ns
	character(len=50) :: fn
	
	integer :: k,lk

        real :: rec_x, rec_y, rec_z

	!======================================================================
	! write displacement fields to files
	!======================================================================
	
	call int2str(my_rank,dummy)
	call int2str(n,ns)
	
	fn=ofd(1:len_trim(ofd))//'/OUTPUT/'//'ux_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
	open(unit=10,file=fn,action='write',form='unformatted')
	fn=ofd(1:len_trim(ofd))//'/OUTPUT/'//'uy_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
	open(unit=11,file=fn,action='write',form='unformatted')
	fn=ofd(1:len_trim(ofd))//'/OUTPUT/'//'uz_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
	open(unit=12,file=fn,action='write',form='unformatted')
	
	write(10) ux
	write(11) uy
	write(12) uz
	
	close(unit=10)
	close(unit=11)
	close(unit=12)

!	fn=ofd(1:len_trim(ofd))//'preco_rw_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
!        !fn='DATA/OUTPUT/preco_rw_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
!        open(unit=13,file=fn,action='write',form='unformatted')
!        write(13) preco
!        close(unit=13)

     
end subroutine ses3d_output


subroutine ses3d_output_grad(n)
use parameters
use variables
implicit none
include 'mpif.h'

	!======================================================================
	! local variables
	!======================================================================
	
	integer, intent(in) :: n 
	character(len=10) :: dummy, ns
	character(len=50) :: fn
	
	!======================================================================
	! write displacement fields to files
	!======================================================================
	
	call int2str(my_rank,dummy)
	call int2str(n,ns)
	
	!fn='../DATA/OUTPUT/grad1_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
	!fn=ofd(1:len_trim(ofd))//'grad1_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
	!open(unit=10,file=fn,action='write',form='unformatted')
	!fn='DATA/OUTPUT/grad2_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
	!open(unit=11,file=fn,action='write',form='unformatted')
	!fn='DATA/OUTPUT/grad3_'//dummy(1:len_trim(dummy))//'_'//ns(1:len_trim(ns))
	!open(unit=12,file=fn,action='write',form='unformatted')
	
	!write(10) grad1
	!write(11) grad2
	!write(12) grad3
	
	!close(unit=10)
	!close(unit=11)
	!close(unit=12)

 
end subroutine ses3d_output_grad
