%function p=z_read_seismograms_auto(array,spacing,component,station,mode)

% function p=z_read_seismograms(array,spacing)
%
% reads seismograms out of the output files and plots them
%
% INPUT:
%
% array:    array of the indeces of the seismograms that will be plotted,
%           enter [] if all seismograms are to be plotted
%
% spacing:  spacing of seismograms in the plot as multiple of the maximum
%           amplitude. A reasonable value is 2.0 .
%
% mode:     'plot' for plotting, otherwise 'silent'
%
% OUTPUT:   a seismogram structure
%
% last modified: 1 May, 2008 (HI)

clear
close all
addpath /l1/heiner/Bin

% Simplue GUI with defaults
dlg_title = 'Parameters for plotting';
prompt = {'Filter ?','Corner frequency (Hz)','Filter order', 'Normalization?','Seismogram spacing','Source (x) ','Source (y)','Sort for distance?'};
num_lines=1;
def ={'0','1','2','1','1','90','0','1'};
answer  = inputdlg(prompt,dlg_title,num_lines,def);
ifilt=str2num(char(answer(1)));
fup=str2num(char(answer(2)));
ord=str2num(char(answer(3)));
inorm=str2num(char(answer(4)));
spacing=str2num(char(answer(5)));
sx=str2num(char(answer(6)));
sy=str2num(char(answer(7)));
isort=str2num(char(answer(8)));

% End GUI



% Read receiver number, names, and coordinates

for ic=1:3,

    fid1=fopen('../MAIN/recfile','r');
    p.nr=fscanf(fid1,'%i',5);

    p.r_x=zeros(1,p.nr);
    p.r_y=zeros(1,p.nr);
    p.r_z=zeros(1,p.nr);
    p.t0=0.0;



    if ic==1,component='x';end
    if ic==2,component='y';end
    if ic==3,component='z';end


    for ir=1:p.nr,


        if ir<10,
            station=fscanf(fid1,'%c',4);
        else
            station=fscanf(fid1,'%c',5);
        end
            
            
        fscanf(fid1,'%g %g %g');

        filename=['../DATA/OUTPUT/' station '_' component];
        disp(sprintf(' Reading from %s ',filename))
        fid=fopen(filename,'r');

        fgets(fid);
        fgets(fid);
        fscanf(fid,'%c',5);
        p.nt=fscanf(fid,'%d',1);
        fscanf(fid,'%c',5);
        p.dt=fscanf(fid,'%g',1);

        fscanf(fid,'%c',4);
        p.r_x(ir)=fscanf(fid,'%g',1);
        p.r_x(ir)=180*p.r_x(ir)/pi;
        fscanf(fid,'%c',4);
        p.r_y(ir)=fscanf(fid,'%g',1);
        p.r_y(ir)=180*p.r_y(ir)/pi;
        fscanf(fid,'%c',4);
        p.r_z(ir)=fscanf(fid,'%g',1);

        % calculate epicentral distance
        p.r_dist(ir)=sqrt((p.r_x(ir)-sx)^2+(p.r_y(ir)-sy)^2);

        for it=1:p.nt
            p.seismograms(ir,it)=fscanf(fid,'%g',1);
        end

        if ifilt==1,
            % tapering and filtering
            p.seismograms(ir,:)=taper(p.seismograms(ir,:)',-250);
            p.seismograms(ir,:)=lowpass(p.seismograms(ir,:),p.dt,fup,ord);
            if inorm==1,
                p.seismograms(ir,:)=p.seismograms(ir,:)/max(p.seismograms(ir,:));
            end
        end


        fclose(fid);

    end


    if isort==1,
        [junk ii]=sort(p.r_dist);
    end




    %==========================================================================
    % plot seismograms
    %==========================================================================


    m=max(max(abs(p.seismograms)));
    t=0:p.dt:((p.nt-1)*p.dt);

    figure
    hold on


    for ir=1:p.nr
        plot(t,ir*spacing*m+p.seismograms(ii(ir),:),'b');
        text(1.05*t(end),ir*spacing*m,num2str(ii(ir)));
        % text(-0.17*t(end),ir*spacing*m+0.1*m,num2str(p.r_x(ir)));
        % text(-0.09*t(end),ir*spacing*m+0.1*m,num2str(p.r_y(ir)));
        % text(-0.01*t(end),ir*spacing*m+0.1*m,num2str(p.r_z(ir)/1000));
    end

    axis([-0.01*t(end) 1.01*t(end) -Inf Inf]);
    grid on


 %   text(-0.17*t(end), p.nr*spacing*m+1.1*m,'\theta [°]');
 %   text(-0.09*t(end), p.nr*spacing*m+1.1*m,'\phi [°]');
 %   text(-0.01*t(end), p.nr*spacing*m+1.1*m,'z [km]');

    xlabel('time [s]');

    if (strcmp(component,'x'))

        ylabel('\theta displacement [m]');
        print -djpeg100 seismograms_x.jpg

    elseif (strcmp(component,'y'))

        ylabel('\phi displacement [m]');
        print -djpeg100 seismograms_y.jpg
        
    elseif (strcmp(component,'z'))

        ylabel('z displacement [m]'); 
        print -djpeg100 seismograms_z.jpg
        

    end


    fclose(fid1);

end


figure

eps=.005;
for ir=1:p.nr,
    
    hold on
    plot(p.r_x(ir),p.r_y(ir),'+')
    text(p.r_x(ir)+eps,p.r_y(ir)+eps,sprintf('%i',ir))
    plot(sx,sy,'rO')
    hold on
    
end
xlabel(' theta (deg) ')
ylabel(' phi (deg) ')
    
print -djpeg100 stations.jpg
    

