!=============================================================================
! model generation for ses3d
!=============================================================================

program generate_models
use variables_new
implicit none
include 'mpif.h'

	!======================================================================
	! initialise mpi and open logfile
	!======================================================================

	call MPI_Init(ierr)
	call MPI_Comm_size(MPI_COMM_WORLD,p,ierr)
	call MPI_Comm_rank(MPI_COMM_WORLD,my_rank,ierr)

	call int2str(my_rank,junk)
	open(unit=99,file='logfile'//junk(1:len_trim(junk)),action='write')
	
	!======================================================================
	! process 0 reads input and broadcasts it to the other processes
	!======================================================================

	if (my_rank==0) then

		open(unit=10,file='model_parameters',action='READ')
	
		read(10,*) junk
		
		read(10,*) nx
		write(99,*) 'total number of elements in x-direction:', nx
		read(10,*) ny
		write(99,*) 'total number of elements in y-direction:', ny
		read(10,*) nz
		write(99,*) 'total number of elements in z-direction:', nz
		read(10,*) lpd
		write(99,*) 'LAGRANGE polynomial degree:', lpd
		read(10,*) x_min
		write(99,*) 'global minimum x value:', x_min
		read(10,*) x_max
		write(99,*) 'global maximum x value:', x_max
		read(10,*) y_min
		write(99,*) 'global minimum y value:', y_min
		read(10,*) y_max
		write(99,*) 'global maximum y value:', y_max
		read(10,*) z_min
		write(99,*) 'global minimum z value:', z_min
		read(10,*) z_max
		write(99,*) 'global maximum z value:', z_max
	
		read(10,*) junk
		read(10,*) model_type
		write(99,*) 'model type:', model_type
		read(10,*) mu_homogen
		write(99,*) 'homogeneous mu:', mu_homogen
		read(10,*) lambda_homogen
		write(99,*) 'homogeneous lambda:', lambda_homogen
		read(10,*) rho_homogen
		write(99,*) 'homogeneous rho:', rho_homogen
		read(10,*) A_homogen
		write(99,*) 'homogeneous A:', A_homogen
		read(10,*) B_homogen
		write(99,*) 'homogeneous B:', B_homogen
		read(10,*) C_homogen
		write(99,*) 'homogeneous C:', C_homogen
		read(10,*) is_aniso
		write(99,*) 'is anisotropic:', is_aniso
		read(10,*) is_formatted
		write(99,*) 'formatted output:', is_formatted
	
		read(10,*) junk
		read(10,*) px
		write(99,*) 'number of processors in x direction:', px
		read(10,*) py
		write(99,*) 'number of processors in y direction:', py
		read(10,*) pz
		write(99,*) 'number of processors in z direction:', pz
	
		close(unit=10)

                x_min=x_min*pi/180
                x_max=x_max*pi/180

                y_min=y_min*pi/180
                y_max=y_max*pi/180
		
	endif
	
	call mpi_bcast(nx,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(ny,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(nz,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	
	call MPI_BCAST(lpd,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	
	call MPI_BCAST(x_min,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(x_max,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(y_min,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(y_max,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(z_min,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(z_max,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	
	call MPI_BCAST(model_type,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(mu_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(lambda_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(rho_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(A_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(B_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(C_homogen,1,MPI_REAL,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(is_aniso,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(is_formatted,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	
	call MPI_BCAST(px,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(py,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)
	call MPI_BCAST(pz,1,MPI_INTEGER,0,MPI_COMM_WORLD,ierr)

        !======================================================================
	! determine collocation points (knots)
	!======================================================================
	
	if (lpd==2) then
		
		knots(0)=-1.0
		knots(1)=0.0
		knots(2)=1.0
		
	elseif (lpd==3) then
		
		knots(0)=-1.0
		knots(1)=-0.4472135954999579
		knots(2)=0.4472135954999579
		knots(3)=1.0
		
	elseif (lpd==4) then
		
		knots(0)=-1.0
		knots(1)=-0.6546536707079772
		knots(2)=0.0
		knots(3)=0.6546536707079772
		knots(4)=1.0
		
	elseif (lpd==5) then
		
		knots(0)=-1.0
		knots(1)=-0.7650553239294647
		knots(2)=-0.2852315164806451
		knots(3)=0.2852315164806451
		knots(4)=0.7650553239294647
		knots(5)=1.0
		
	elseif (lpd==6) then
		
		knots(0)=-1.0
		knots(1)=-0.8302238962785670
		knots(2)=-0.4688487934707142
		knots(3)=0.0
		knots(4)=0.4688487934707142
		knots(5)=0.8302238962785670
		knots(6)=1.0
		
	elseif (lpd==7) then
		
		knots(0)=-1.0
		knots(1)=-0.8717401485096066
		knots(2)=-0.5917001814331423
		knots(3)=-0.2092992179024789
		knots(4)=0.2092992179024789
		knots(5)=0.5917001814331423
		knots(6)=0.8717401485096066
		knots(7)=1.0
		
	endif


	!======================================================================
	! make boxfile containing the parameters for parallel computing
	!======================================================================

	if (my_rank==0) then

		write(*,*) 'writing boxfile'
	
	endif

	allocate(min_index(1:px*py*pz,1:3),STAT=status)		! minimum element indeces
	allocate(max_index(1:px*py*pz,1:3),STAT=status)		! maximum element indeces
	
	dz=(z_max-z_min)/(nz+pz)					! width of the elements in z direction
	dy=(y_max-y_min)/(ny+py)					! width of the elements in y direction
	dx=(x_max-x_min)/(nx+px)					! width of the elements in x direction
	
	bnx=floor(real((nx+1)/px))				! number of elements per processor in x direction
	bny=floor(real((ny+1)/py))				! number of elements per processor in y direction
	bnz=floor(real((nz+1)/pz))				! number of elements per processor in z direction

	if (my_rank==0) then
	
		open(unit=11,file='boxfile',action='WRITE')
		
		write(11,*) "- format description -"
		write(11,*) "total number of processes"
		write(11,*) "number of processes in x direction"
		write(11,*) "number of processes in y direction"
		write(11,*) "number of processes in z direction"
		write(11,*) "single index (npz-1)*py*px+(npy-1)*px+npx"
		write(11,*) "multi index (npx, npy, npz)"
		write(11,*) "index boundaries x (x_min, x_max)"
		write(11,*) "index boundaries y (y_min, y_max)"
		write(11,*) "index boundaries z (z_min, z_max)"
		write(11,*) "physical boundaries x"
		write(11,*) "physical boundaries y"
		write(11,*) "phyiscal boundaries z"
	
		write(11,*) '-------------------'
		write(11,*) px*py*pz
		write(11,*) px
		write(11,*) py
		write(11,*) pz
		write(11,*) '-------------------'
		
	endif
	
	do k=1,pz,1
		do j=1,py,1
			do i=1,px,1
				
				! indices (multiple and single) of the processor boxes (mi=my_rank+1)
				mi=(k-1)*py*px+(j-1)*px+i	
		
				if (my_rank==0) then
				
					write(11,*) mi
					write(11,*) i,j,k
					
				endif
				
				! index boundaries of the processor boxes
				min_index(mi,1)=(i-1)*bnx
				min_index(mi,2)=(j-1)*bny
				min_index(mi,3)=(k-1)*bnz
				
				if (i/=px) then
					max_index(mi,1)=i*bnx
				else
					max_index(mi,1)=nx
				endif
				
				if (j/=py) then
					max_index(mi,2)=j*bny
				else
					max_index(mi,2)=ny
				endif
				
				if (k/=pz) then
					max_index(mi,3)=k*bnz
				else
					max_index(mi,3)=nz
				endif
				
				if (my_rank==0) then
				
					write(11,*) min_index(mi,1), max_index(mi,1)
					write(11,*) min_index(mi,2), max_index(mi,2)
					write(11,*) min_index(mi,3), max_index(mi,3)
				
					! physical boundaries of the processor boxes
					write(11,*) x_min+(min_index(mi,1)+i-1)*dx, x_min+(max_index(mi,1)+i)*dx
					write(11,*) y_min+(min_index(mi,2)+j-1)*dy, y_min+(max_index(mi,2)+j)*dy
					write(11,*) z_min+(min_index(mi,3)+k-1)*dz, z_min+(max_index(mi,3)+k)*dz
					write(11,*) '-------------------'
					
				endif
			
				if (my_rank==(mi-1)) then
					
					nx_loc=max_index(my_rank+1,1)-min_index(my_rank+1,1)
					ny_loc=max_index(my_rank+1,2)-min_index(my_rank+1,2)
					nz_loc=max_index(my_rank+1,3)-min_index(my_rank+1,3)

					x_min_loc=x_min+(min_index(my_rank+1,1)+i-1)*dx
					x_max_loc=x_min+(max_index(my_rank+1,1)+i)*dx

					y_min_loc=y_min+(min_index(my_rank+1,2)+j-1)*dy
					y_max_loc=y_min+(max_index(my_rank+1,2)+j)*dy

					z_min_loc=z_min+(min_index(my_rank+1,3)+k-1)*dz
					z_max_loc=z_min+(max_index(my_rank+1,3)+k)*dz
					
				endif
					
			enddo
		enddo
	enddo
				
	if (my_rank==0) then
	
		close(UNIT=11)
	
	endif

	
        !======================================================================
        ! make coordinates
        !======================================================================

        write(99,*) '- indices ----------------------'
        write(99,*) nx_loc, ny_loc, nz_loc
        write(99,*) '- phys. model dimensions -------'
        write(99,*) x_min_loc*180/pi, x_max_loc*180/pi
        write(99,*) y_min_loc*180/pi, y_max_loc*180/pi
        write(99,*) z_min_loc, z_max_loc

        allocate(x(0:nx_loc,0:lpd))
        allocate(y(0:ny_loc,0:lpd))
        allocate(z(0:nz_loc,0:lpd))
	
	allocate(XX(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(YY(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(ZZ(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))

        do i=0,nx_loc
		do n=0,lpd
			x(i,n)=x_min_loc+i*dx+0.5*(1+knots(n))*dx
			XX(i,:,:,n,:,:)=x_min_loc+i*dx+0.5*(1+knots(n))*dx
		enddo
	enddo

	do j=0,ny_loc
		do n=0,lpd
			y(j,n)=y_min_loc+j*dy+0.5*(1+knots(n))*dy
			YY(:,j,:,:,n,:)=y_min_loc+j*dy+0.5*(1+knots(n))*dy
		enddo
	enddo
	
	do k=0,nz_loc
		do n=0,lpd
			z(k,n)=z_max_loc-k*dz-0.5*(1+knots(n))*dz
			ZZ(:,:,k,:,:,n)=z_max_loc-k*dz-0.5*(1+knots(n))*dz
		enddo
	enddo
	
	XX=XX*180/pi
	YY=YY*180/pi
 
        do i=0,nz_loc
           write(99,*) z(i,:)
        enddo

	!======================================================================
	! allocate memory
	!======================================================================

	allocate(rhoinv(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(mu(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(lambda(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(A(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(B(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(C(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))

        allocate(rho(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
        allocate(cs(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
        allocate(cp(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
        

		!=============================================================
		! homogeneous model
		!=============================================================

		if (model_type==1) then
			
                       call homogeneous
			
		endif
		
		!=============================================================
		! PREM isotropic (without ocean but with crust)
		!=============================================================
		
		if (model_type==3) then

			call prem_iso_no_ocean
                    
                endif
		
		!=============================================================
		! PREM + hetero
		!=============================================================
		
		if (model_type==31) then
			
			call prem_iso_hetero1
		
		endif
				
		!=============================================================
		! AK135
		!=============================================================
		
		if (model_type==7) then

                    call ak135
							
                endif
		
		!=============================================================
		! AK135, modifiziert
		!=============================================================
		
		if (model_type==71) then
			
			call ak135_mod2
		
		endif
	
		!=============================================================
		! modified AK135 (55km crust)
		!=============================================================
		
		if (model_type==72) then

                    call ak135_mod
							
                endif
		
		!==============================================================
		! AK135 + heterogeneity
		!==============================================================
		
		if (model_type==73) then
			
			call ak135_3d
		
		endif
				
		!=============================================================
		! AUSMOD
		!=============================================================
	
		if (model_type==100) then

                        done=0
			
			if (my_rank==0) then
				
				write(*,*) 'AUSMOD'

			endif
			
			call ses3d_read_ausmod

                endif

		!=============================================================
		! AUSMOD_2
		!=============================================================
	
		if (model_type==101) then

                        done=0
			
			if (my_rank==0) then
				
				write(*,*) 'AUSMOD'

			endif
			
			call ses3d_read_ausmod2

                endif

		!=============================================================
		! SPICE - TOTTORI
		!=============================================================
	
		if (model_type==200) then

			if (my_rank==0) then
				
				write(*,*) 'SPICE Tottori'

			endif
			
			call spice_tottori

                endif




			
	!======================================================================
	! save to files
	!======================================================================

		!==============================================================
		! open files, formatted or unformatted
		!==============================================================

		call int2str(my_rank,junk)
			
		OPEN(UNIT=11,FILE='rhoinv'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
		OPEN(UNIT=12,FILE='mu'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
		OPEN(UNIT=13,FILE='lambda'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
		
		if (is_aniso==1) then
				
			OPEN(UNIT=14,FILE='A'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
			OPEN(UNIT=15,FILE='B'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
			OPEN(UNIT=16,FILE='C'//junk(1:len_trim(junk)),ACTION='WRITE',form='unformatted')
				
		endif
		
		!==============================================================
		! write parameters to files, formatted or unformatted
		!==============================================================
			
		write(11) rhoinv(:,:,:,:,:,:)
		write(12) mu(:,:,:,:,:,:)
		write(13) lambda(:,:,:,:,:,:)
			
		if (is_aniso==1) then
				
			write(14) A(:,:,:,:,:,:)
			write(15) B(:,:,:,:,:,:)
			write(16) C(:,:,:,:,:,:)
				
		endif
				
		close(UNIT=11)
		close(UNIT=12)
		close(UNIT=13)
		
		if (is_aniso==1) then
		
			close(UNIT=14)
			close(UNIT=15)
			close(UNIT=16)
			
		endif

	!======================================================================
	! make a profile
	!======================================================================
	
	
	OPEN(UNIT=11,FILE='prof_rhoinv'//junk(1:len_trim(junk)),ACTION='WRITE')
	OPEN(UNIT=12,FILE='prof_lambda'//junk(1:len_trim(junk)),ACTION='WRITE')
	OPEN(UNIT=13,FILE='prof_mu'//junk(1:len_trim(junk)),ACTION='WRITE')


        do i=0,nz_loc
        	do k=0,lpd
		
			write(11,*) rhoinv(0,0,i,0,0,k), z(i,k)
			write(12,*) lambda(0,0,i,0,0,k), z(i,k)
			write(13,*) mu(0,0,i,0,0,k), z(i,k)
			
		enddo
	enddo

	close(unit=11)
	close(unit=12)
	close(unit=13)

	!=======================================================================
	! clean up
	!=======================================================================
	
	deallocate(rhoinv)
	deallocate(mu)
	deallocate(lambda)
	
	deallocate(A)
	deallocate(B)
	deallocate(C)

        deallocate(rho)
        deallocate(cp)
        deallocate(cs)
	
	deallocate(min_index)
	deallocate(max_index)

        deallocate(x)
        deallocate(y)
        deallocate(z)
	
	deallocate(XX)
	deallocate(YY)
	deallocate(ZZ)
	
	close(unit=99)
	call MPI_Finalize(ierr)

end program generate_models



!==============================================================================
! subroutine for integer to string conversion
!==============================================================================

subroutine int2str(value, string)
implicit none

	integer, intent(in) :: value
	character(len=*), intent(inout) :: string
	
	character(len=10) :: c
	integer :: k, n, new_value, is
	real :: e
	
	
	e=1e9
	is=0
	
	if (value==0) then
		string(1:1)='0'
		string(2:10)=' '
	else
	
		new_value=value
	
		do k=1,10
			c(k:k)=char(floor(new_value/e)+48)
		
			if ((floor(new_value/e)==0) .and. (is==0)) then
				n=k
			else
				is=1
			endif
		
			!write(*,*) c
			new_value=new_value-e*floor(new_value/e)
			e=e/10
			string(k:k)=' '
		
		enddo
		
		string(1:10-n)=c(n+1:10)
		
	endif

	if (len(string)>10) then
		string(11:len(string))=' '
	endif

return
end subroutine int2str
