!===================================================================================
! reads the AUSMOD model for model generation in ses3d
! last modified: 8 April, 2008, Andreas Fichtner
!===================================================================================

subroutine ses3d_read_ausmod2
use variables_new
implicit none
include 'mpif.h'

	!===========================================================================
	! variables and parameters
	!===========================================================================
			
        integer :: i_d, i_lat, i_lon, kix
	integer :: res
                        		
	!- geometry ----------------------------------------------------------------
					
	real :: lon(0:500), lat(0:300), d(0:24)

	!- values ------------------------------------------------------------------
        
	real :: in_lon, in_lat, in_d
	real, dimension(0:500,0:300,0:24) :: rho_am, mu_am, lambda_am, cp_am, cs_am
	
	!===========================================================================
	! initialisation of AUSMOD geometry
	!===========================================================================
	
	do kix=0,500
		lon(kix)=77.5+0.25*kix
	enddo
	
	do kix=0,300
		lat(kix)=77.5+0.25*kix
	enddo
	
	d=(/6371.0, 6371.0-2.5, 6371.0-5.0, 6371.0-7.5, 6371.0-10.0, 6371.0-15.0, 6371.0-20.0, &
	    6371.0-25.0, 6371.0-30.0, 6371.0-35.0, 6371.0-40.0, 6371.0-50.0, 6371.0-55.0, 6371.0-75.0, &
	    6371.0-100.0, 6371.0-125.0, 6371.0-150.0, 6371.0-175.0, 6371.0-200.0, 6371.0-225.0, &
	    6371.0-250.0, 6371.0-275.0, 6371.0-300.0, 6371.0-325.0, 6371.0-350.0/)
	    
	d=1000*d

	
		
	!===========================================================================
	! make 1D background - PREM without ocean
	!===========================================================================
			
	call prem_iso_no_ocean

	cp=sqrt((lambda+2*mu)*rhoinv)
	cs=sqrt(mu*rhoinv)

	!===========================================================================
	! read files
	!===========================================================================

	open(unit=611,file='AUSMOD/ausmod-dcp-smooth100',action='read')
	open(unit=612,file='AUSMOD/ausmod-dcs-smooth100',action='read')

        do m=0,24
	do n=0,500
        do i_lat=0,300
	
			read(611,*) cp_am(n,i_lat,m)
			read(612,*) cs_am(n,i_lat,m)

        enddo
	enddo
	enddo
	
	close(unit=611)
	close(unit=612)

        cp_am=1000*cp_am
        cs_am=1000*cs_am
	
	!===========================================================================
	! computations
	!===========================================================================

	!- 3D part ---------------------------------------------------------

	res=3
	
	
	do i_lon=res,500,res
	
		if (my_rank==0) then
			write(*,*) i_lon, lon(i_lon)
		endif
	
		do i_lat=res,300,res
		do i_d=1,24,1
	
			!- only when the interval is inside the processor box -----
		
			if ((lat(i_lat)>=x_min_loc*180/pi) .and. (lat(i_lat-res)<=x_max_loc*180/pi) .and. &
			    (lon(i_lon)>=y_min_loc*180/pi) .and. (lon(i_lon-res)<=y_max_loc*180/pi) .and. &
			    (d(i_d-1)>=z_min_loc) .and. (d(i_d)<=z_max_loc)) then
		
				where((XX<lat(i_lat)) .and. (XX>=lat(i_lat-res)) .and. &
				(YY<lon(i_lon)) .and. (YY>=lon(i_lon-res)) .and. &
				(ZZ>d(i_d)) .and. (ZZ<=d(i_d-1)))

                                   cp=cp+cp_am(i_lon-res,i_lat-res,i_d-1)
				   cs=cs+cs_am(i_lon-res,i_lat-res,i_d-1)
		      
				end where
				
			endif
	
		enddo
		enddo
        enddo

	!===========================================================================
	! Homogenisierung
	!===========================================================================
	
	!allocate(oci(0:nx_loc,0:ny_loc,0:nz_loc,0:lpd,0:lpd,0:lpd))
	allocate(oci(0:nx_loc,0:ny_loc,0:lpd,0:lpd))

	oci(:,:,:,:)=(cs(:,:,0,:,:,0)-3200.0)/1000;

	cs(:,:,0,:,:,0)=1000*(3.20*oci(:,:,:,:)-2.50*(oci(:,:,:,:)-1))
	cp(:,:,0,:,:,0)=1000*(5.78*oci(:,:,:,:)-5.00*(oci(:,:,:,:)-1))
	rho(:,:,0,:,:,0)=1000*(2.60*oci(:,:,:,:)-2.60*(oci(:,:,:,:)-1))

	cs(:,:,0,:,:,1)=1000*(3.18*oci(:,:,:,:)-4.13*(oci(:,:,:,:)-1))
	cp(:,:,0,:,:,1)=1000*(5.77*oci(:,:,:,:)-7.44*(oci(:,:,:,:)-1))
	rho(:,:,0,:,:,1)=1000*(2.68*oci(:,:,:,:)-3.21*(oci(:,:,:,:)-1))

	cs(:,:,0,:,:,2)=1000*(3.72*oci(:,:,:,:)-4.56*(oci(:,:,:,:)-1))
	cp(:,:,0,:,:,2)=1000*(6.54*oci(:,:,:,:)-8.19*(oci(:,:,:,:)-1))
	rho(:,:,0,:,:,2)=1000*(2.84*oci(:,:,:,:)-3.43*(oci(:,:,:,:)-1))

	cs(:,:,0,:,:,3)=1000*(4.02*oci(:,:,:,:)-4.41*(oci(:,:,:,:)-1))
	cp(:,:,0,:,:,3)=1000*(6.98*oci(:,:,:,:)-7.99*(oci(:,:,:,:)-1))
	rho(:,:,0,:,:,3)=1000*(2.92*oci(:,:,:,:)-3.30*(oci(:,:,:,:)-1))

	cs(:,:,0,:,:,4)=1000*(3.89*oci(:,:,:,:)-4.49*(oci(:,:,:,:)-1))
	cp(:,:,0,:,:,4)=1000*(6.80*oci(:,:,:,:)-8.10*(oci(:,:,:,:)-1))
	rho(:,:,0,:,:,4)=1000*(2.90*oci(:,:,:,:)-3.38*(oci(:,:,:,:)-1))

	cs(:,:,1,:,:,0)=1000*(4.48*oci(:,:,:,:)-4.48*(oci(:,:,:,:)-1))
	cp(:,:,1,:,:,0)=1000*(8.10*oci(:,:,:,:)-8.10*(oci(:,:,:,:)-1))
	rho(:,:,1,:,:,0)=1000*(3.38*oci(:,:,:,:)-3.38*(oci(:,:,:,:)-1))

	deallocate(oci)

	!===========================================================================
	! make mu, lambda
	!===========================================================================

	do i=0,nz_loc
        	do k=0,lpd

                        mu(:,:,i,:,:,k)=cs(:,:,i,:,:,k)*cs(:,:,i,:,:,k)/rhoinv(:,:,i,:,:,k)
                        lambda(:,:,i,:,:,k)=(cp(:,:,i,:,:,k)*cp(:,:,i,:,:,k) &
			  		-2*cs(:,:,i,:,:,k)*cs(:,:,i,:,:,k))/rhoinv(:,:,i,:,:,k)

                enddo
        enddo


                
		
end subroutine ses3d_read_ausmod2
